@@ -1,13 +1,13 @@
 package hll
 
 import (
-	crand "crypto/rand"
 	"encoding/binary"
 	"fmt"
 	"math"
 	"math/rand"
 	"reflect"
 	"testing"
+	"unsafe"
 
 	"github.com/davecgh/go-spew/spew"
 )
@@ -25,7 +25,51 @@ func toByte(v uint64) []byte {
 	return buf[:]
 }
 
-func TestHLLPP_Add_NoSparse(t *testing.T) {
+func TestPlus_Bytes(t *testing.T) {
+	testCases := []struct {
+		p      uint8
+		normal bool
+	}{
+		{4, false},
+		{5, false},
+		{4, true},
+		{5, true},
+	}
+
+	for i, testCase := range testCases {
+		t.Run(fmt.Sprint(i), func(t *testing.T) {
+			h := NewTestPlus(testCase.p)
+
+			plusStructOverhead := int(unsafe.Sizeof(*h))
+			compressedListOverhead := int(unsafe.Sizeof(*h.sparseList))
+
+			var expectedDenseListCapacity, expectedSparseListCapacity int
+
+			if testCase.normal {
+				h.toNormal()
+				// denseList has capacity for 2^p elements, one byte each
+				expectedDenseListCapacity = int(math.Pow(2, float64(testCase.p)))
+				if expectedDenseListCapacity != cap(h.denseList) {
+					t.Errorf("denseList capacity: want %d got %d", expectedDenseListCapacity, cap(h.denseList))
+				}
+			} else {
+				// sparseList has capacity for 2^p elements, one byte each
+				expectedSparseListCapacity = int(math.Pow(2, float64(testCase.p)))
+				if expectedSparseListCapacity != cap(h.sparseList.b) {
+					t.Errorf("sparseList capacity: want %d got %d", expectedSparseListCapacity, cap(h.sparseList.b))
+				}
+				expectedSparseListCapacity += compressedListOverhead
+			}
+
+			expectedSize := plusStructOverhead + expectedDenseListCapacity + expectedSparseListCapacity
+			if expectedSize != h.Bytes() {
+				t.Errorf("Bytes(): want %d got %d", expectedSize, h.Bytes())
+			}
+		})
+	}
+}
+
+func TestPlus_Add_NoSparse(t *testing.T) {
 	h := NewTestPlus(16)
 	h.toNormal()
 
@@ -66,7 +110,7 @@ func TestHLLPP_Add_NoSparse(t *testing.T) {
 	}
 }
 
-func TestHLLPPPrecision_NoSparse(t *testing.T) {
+func TestPlusPrecision_NoSparse(t *testing.T) {
 	h := NewTestPlus(4)
 	h.toNormal()
 
@@ -89,7 +133,7 @@ func TestHLLPPPrecision_NoSparse(t *testing.T) {
 	}
 }
 
-func TestHLLPP_toNormal(t *testing.T) {
+func TestPlus_toNormal(t *testing.T) {
 	h := NewTestPlus(16)
 	h.Add(toByte(0x00010fffffffffff))
 	h.toNormal()
@@ -131,7 +175,7 @@ func TestHLLPP_toNormal(t *testing.T) {
 	}
 }
 
-func TestHLLPPCount(t *testing.T) {
+func TestPlusCount(t *testing.T) {
 	h := NewTestPlus(16)
 
 	n := h.Count()
@@ -166,7 +210,7 @@ func TestHLLPPCount(t *testing.T) {
 	}
 }
 
-func TestHLLPP_Merge_Error(t *testing.T) {
+func TestPlus_Merge_Error(t *testing.T) {
 	h := NewTestPlus(16)
 	h2 := NewTestPlus(10)
 
@@ -266,7 +310,7 @@ func TestHLL_Merge_Normal(t *testing.T) {
 	}
 }
 
-func TestHLLPP_Merge(t *testing.T) {
+func TestPlus_Merge(t *testing.T) {
 	h := NewTestPlus(16)
 
 	k1 := uint64(0xf000017000000000)
@@ -359,7 +403,7 @@ func TestHLLPP_Merge(t *testing.T) {
 	}
 }
 
-func TestHLLPP_EncodeDecode(t *testing.T) {
+func TestPlus_EncodeDecode(t *testing.T) {
 	h := NewTestPlus(8)
 	i, r := h.decodeHash(h.encodeHash(0xffffff8000000000))
 	if i != 0xff {
@@ -402,7 +446,7 @@ func TestHLLPP_EncodeDecode(t *testing.T) {
 	}
 }
 
-func TestHLLPP_Error(t *testing.T) {
+func TestPlus_Error(t *testing.T) {
 	_, err := NewPlus(3)
 	if err == nil {
 		t.Error("precision 3 should return error")
@@ -419,14 +463,16 @@ func TestHLLPP_Error(t *testing.T) {
 	}
 }
 
-func TestHLLPP_Marshal_Unmarshal_Sparse(t *testing.T) {
+func TestPlus_Marshal_Unmarshal_Sparse(t *testing.T) {
 	h, _ := NewPlus(4)
 	h.sparse = true
-	h.tmpSet = map[uint32]struct{}{26: struct{}{}, 40: struct{}{}}
+	h.tmpSet = map[uint32]struct{}{26: {}, 40: {}}
+
+	src := rand.New(rand.NewSource(6611))
 
 	// Add a bunch of values to the sparse representation.
 	for i := 0; i < 10; i++ {
-		h.sparseList.Append(uint32(rand.Int()))
+		h.sparseList.Append(uint32(src.Int()))
 	}
 
 	data, err := h.MarshalBinary()
@@ -452,13 +498,15 @@ func TestHLLPP_Marshal_Unmarshal_Sparse(t *testing.T) {
 	}
 }
 
-func TestHLLPP_Marshal_Unmarshal_Dense(t *testing.T) {
+func TestPlus_Marshal_Unmarshal_Dense(t *testing.T) {
 	h, _ := NewPlus(4)
 	h.sparse = false
 
+	src := rand.New(rand.NewSource(1688))
+
 	// Add a bunch of values to the dense representation.
 	for i := 0; i < 10; i++ {
-		h.denseList = append(h.denseList, uint8(rand.Int()))
+		h.denseList = append(h.denseList, uint8(src.Int()))
 	}
 
 	data, err := h.MarshalBinary()
@@ -486,7 +534,7 @@ func TestHLLPP_Marshal_Unmarshal_Dense(t *testing.T) {
 
 // Tests that a sketch can be serialised / unserialised and keep an accurate
 // cardinality estimate.
-func TestHLLPP_Marshal_Unmarshal_Count(t *testing.T) {
+func TestPlus_Marshal_Unmarshal_Count(t *testing.T) {
 	if testing.Short() {
 		t.Skip("Skipping test in short mode")
 	}
@@ -494,9 +542,11 @@ func TestHLLPP_Marshal_Unmarshal_Count(t *testing.T) {
 	count := make(map[string]struct{}, 1000000)
 	h, _ := NewPlus(16)
 
+	src := rand.New(rand.NewSource(6828))
+
 	buf := make([]byte, 8)
 	for i := 0; i < 1000000; i++ {
-		if _, err := crand.Read(buf); err != nil {
+		if _, err := src.Read(buf); err != nil {
 			panic(err)
 		}
 
@@ -532,7 +582,7 @@ func TestHLLPP_Marshal_Unmarshal_Count(t *testing.T) {
 
 	// Add some more values.
 	for i := 0; i < 1000000; i++ {
-		if _, err := crand.Read(buf); err != nil {
+		if _, err := src.Read(buf); err != nil {
 			panic(err)
 		}
 
@@ -551,19 +601,22 @@ func TestHLLPP_Marshal_Unmarshal_Count(t *testing.T) {
 }
 
 func NewTestPlus(p uint8) *Plus {
-	h, _ := NewPlus(p)
+	h, err := NewPlus(p)
+	if err != nil {
+		panic(err)
+	}
 	h.hash = nopHash
 	return h
 }
 
 // Generate random data to add to the sketch.
-func genData(n int) [][]byte {
+func genData(n int, src *rand.Rand) [][]byte {
 	out := make([][]byte, 0, n)
 	buf := make([]byte, 8)
 
 	for i := 0; i < n; i++ {
 		// generate 8 random bytes
-		n, err := rand.Read(buf)
+		n, err := src.Read(buf)
 		if err != nil {
 			panic(err)
 		} else if n != 8 {
@@ -582,10 +635,11 @@ func genData(n int) [][]byte {
 var benchdata = map[int][][]byte{}
 
 func benchmarkPlusAdd(b *testing.B, h *Plus, n int) {
+	src := rand.New(rand.NewSource(9938))
 	blobs, ok := benchdata[n]
 	if !ok {
 		// Generate it.
-		benchdata[n] = genData(n)
+		benchdata[n] = genData(n, src)
 		blobs = benchdata[n]
 	}
 
