@@ -2,10 +2,10 @@ package mmap_test
 
 import (
 	"bytes"
-	"io/ioutil"
+	"os"
 	"testing"
 
-	"github.com/influxdata/influxdb/pkg/mmap"
+	"github.com/influxdata/influxdb/v2/pkg/mmap"
 )
 
 func TestMap(t *testing.T) {
@@ -14,8 +14,8 @@ func TestMap(t *testing.T) {
 		t.Fatalf("Open: %v", err)
 	}
 
-	if exp, err := ioutil.ReadFile("mmap_test.go"); err != nil {
-		t.Fatalf("ioutil.ReadFile: %v", err)
+	if exp, err := os.ReadFile("mmap_test.go"); err != nil {
+		t.Fatalf("os.ReadFile: %v", err)
 	} else if !bytes.Equal(data, exp) {
 		t.Fatalf("got %q\nwant %q", string(data), string(exp))
 	}
