@@ -5,8 +5,7 @@ import (
 	"fmt"
 	"time"
 
-	"github.com/influxdata/influxdb/monitor/diagnostics"
-	"github.com/influxdata/influxdb/toml"
+	"github.com/influxdata/influxdb/v2/toml"
 )
 
 const (
@@ -14,7 +13,7 @@ const (
 	DefaultEngine = "tsm1"
 
 	// DefaultIndex is the default index for new shards
-	DefaultIndex = InmemIndexName
+	DefaultIndex = TSI1IndexName
 
 	// tsdb/engine/wal configuration options
 
@@ -52,10 +51,6 @@ const (
 	// block in a TSM file
 	DefaultMaxPointsPerBlock = 1000
 
-	// DefaultMaxSeriesPerDatabase is the maximum number of series a node can hold per database.
-	// This limit only applies to the "inmem" index.
-	DefaultMaxSeriesPerDatabase = 1000000
-
 	// DefaultMaxValuesPerTag is the maximum number of values a tag can have within a measurement.
 	DefaultMaxValuesPerTag = 100000
 
@@ -85,6 +80,16 @@ type Config struct {
 	// General WAL configuration options
 	WALDir string `toml:"wal-dir"`
 
+	// WALMaxConcurrentWrites sets the max number of WAL writes that can be attempted at one time.
+	// In reality only one write to disk can run at a time, but we allow the preceding encoding steps
+	// to run concurrently. This can cause allocations to increase quickly when writing to a slow disk.
+	// Set to 0 to use the default (<nprocs> * 2).
+	WALMaxConcurrentWrites int `toml:"wal-max-concurrent-writes"`
+
+	// WALMaxWriteDelay is the max amount of time the WAL will wait to begin a write when there are
+	// already WALMaxConcurrentWrites in progress. A value of 0 disables any timeout.
+	WALMaxWriteDelay time.Duration `toml:"wal-max-write-delay"`
+
 	// WALFsyncDelay is the amount of time that a write will wait before fsyncing.  A duration
 	// greater than 0 can be used to batch up multiple fsync calls.  This is useful for slower
 	// disks or when WAL write contention is seen.  A value of 0 fsyncs every write to the WAL.
@@ -93,8 +98,8 @@ type Config struct {
 	// Enables unicode validation on series keys on write.
 	ValidateKeys bool `toml:"validate-keys"`
 
-	// Enables strict error handling. For example, forces SELECT INTO to err out on INF values.
-	StrictErrorHandling bool `toml:"strict-error-handling"`
+	// When true, skips size validation on fields
+	SkipFieldSizeValidation bool `toml:"skip-field-size-validation"`
 
 	// Query logging
 	QueryLogEnabled bool `toml:"query-log-enabled"`
@@ -109,16 +114,6 @@ type Config struct {
 
 	// Limits
 
-	// MaxSeriesPerDatabase is the maximum number of series a node can hold per database.
-	// When this limit is exceeded, writes return a 'max series per database exceeded' error.
-	// A value of 0 disables the limit. This limit only applies when using the "inmem" index.
-	MaxSeriesPerDatabase int `toml:"max-series-per-database"`
-
-	// MaxValuesPerTag is the maximum number of tag values a single tag key can have within
-	// a measurement.  When the limit is execeeded, writes return an error.
-	// A value of 0 disables the limit.
-	MaxValuesPerTag int `toml:"max-values-per-tag"`
-
 	// MaxConcurrentCompactions is the maximum number of concurrent level and full compactions
 	// that can be running at one time across all shards.  Compactions scheduled to run when the
 	// limit is reached are blocked until a running compaction completes.  Snapshot compactions are
@@ -138,7 +133,7 @@ type Config struct {
 
 	// SeriesFileMaxConcurrentSnapshotCompactions is the maximum number of concurrent snapshot compactions
 	// that can be running at one time across all series partitions in a database. Snapshots scheduled
-	// to run when the limit is reached are blocked until a running snaphsot completes.  Only snapshot
+	// to run when the limit is reached are blocked until a running snapshot completes.  Only snapshot
 	// compactions are affected by this limit. A value of 0 limits snapshot compactions to the lesser of
 	// 8 (series file partition quantity) and runtime.GOMAXPROCS(0).
 	SeriesFileMaxConcurrentSnapshotCompactions int `toml:"series-file-max-concurrent-snapshot-compactions"`
@@ -158,8 +153,7 @@ func NewConfig() Config {
 		Engine: DefaultEngine,
 		Index:  DefaultIndex,
 
-		StrictErrorHandling: false,
-		QueryLogEnabled:     true,
+		QueryLogEnabled: true,
 
 		CacheMaxMemorySize:             toml.Size(DefaultCacheMaxMemorySize),
 		CacheSnapshotMemorySize:        toml.Size(DefaultCacheSnapshotMemorySize),
@@ -168,10 +162,10 @@ func NewConfig() Config {
 		CompactThroughput:              toml.Size(DefaultCompactThroughput),
 		CompactThroughputBurst:         toml.Size(DefaultCompactThroughputBurst),
 
-		MaxSeriesPerDatabase:     DefaultMaxSeriesPerDatabase,
-		MaxValuesPerTag:          DefaultMaxValuesPerTag,
 		MaxConcurrentCompactions: DefaultMaxConcurrentCompactions,
 
+		WALMaxWriteDelay: 10 * time.Minute,
+
 		MaxIndexLogFileSize:  toml.Size(DefaultMaxIndexLogFileSize),
 		SeriesIDSetCacheSize: DefaultSeriesIDSetCacheSize,
 
@@ -226,23 +220,3 @@ func (c *Config) Validate() error {
 
 	return nil
 }
-
-// Diagnostics returns a diagnostics representation of a subset of the Config.
-func (c Config) Diagnostics() (*diagnostics.Diagnostics, error) {
-	return diagnostics.RowFromMap(map[string]interface{}{
-		"dir":                                    c.Dir,
-		"wal-dir":                                c.WALDir,
-		"wal-fsync-delay":                        c.WALFsyncDelay,
-		"strict-error-handling":                  c.StrictErrorHandling,
-		"cache-max-memory-size":                  c.CacheMaxMemorySize,
-		"cache-snapshot-memory-size":             c.CacheSnapshotMemorySize,
-		"cache-snapshot-write-cold-duration":     c.CacheSnapshotWriteColdDuration,
-		"compact-full-write-cold-duration":       c.CompactFullWriteColdDuration,
-		"max-series-per-database":                c.MaxSeriesPerDatabase,
-		"max-values-per-tag":                     c.MaxValuesPerTag,
-		"max-concurrent-compactions":             c.MaxConcurrentCompactions,
-		"max-index-log-file-size":                c.MaxIndexLogFileSize,
-		"series-id-set-cache-size":               c.SeriesIDSetCacheSize,
-		"series-file-max-concurrent-compactions": c.SeriesFileMaxConcurrentSnapshotCompactions,
-	}), nil
-}
