@@ -11,18 +11,15 @@ import (
 	"sort"
 	"time"
 
-	"github.com/influxdata/influxdb/models"
-	"github.com/influxdata/influxdb/pkg/estimator"
-	"github.com/influxdata/influxdb/pkg/limiter"
-	"github.com/influxdata/influxdb/query"
+	"github.com/influxdata/influxdb/v2/influxql/query"
+	"github.com/influxdata/influxdb/v2/models"
+	"github.com/influxdata/influxdb/v2/pkg/estimator"
+	"github.com/influxdata/influxdb/v2/pkg/limiter"
 	"github.com/influxdata/influxql"
 	"go.uber.org/zap"
 )
 
 var (
-	// ErrFormatNotFound is returned when no format can be determined from a path.
-	ErrFormatNotFound = errors.New("format not found")
-
 	// ErrUnknownEngineFormat is returned when the engine format is
 	// unknown. ErrUnknownEngineFormat is currently returned if a format
 	// other than tsm1 is encountered.
@@ -31,12 +28,15 @@ var (
 
 // Engine represents a swappable storage engine for the shard.
 type Engine interface {
-	Open() error
+	Open(ctx context.Context) error
 	Close() error
 	SetEnabled(enabled bool)
 	SetCompactionsEnabled(enabled bool)
 	ScheduleFullCompaction() error
 
+	SetNewReadersBlocked(blocked bool) error
+	InUse() (bool, error)
+
 	WithLogger(*zap.Logger)
 
 	LoadMetadataIndex(shardID uint64, index Index) error
@@ -51,12 +51,12 @@ type Engine interface {
 	CreateIterator(ctx context.Context, measurement string, opt query.IteratorOptions) (query.Iterator, error)
 	CreateCursorIterator(ctx context.Context) (CursorIterator, error)
 	IteratorCost(measurement string, opt query.IteratorOptions) (query.IteratorCost, error)
-	WritePoints(points []models.Point) error
+	WritePoints(ctx context.Context, points []models.Point) error
 
 	CreateSeriesIfNotExists(key, name []byte, tags models.Tags) error
 	CreateSeriesListIfNotExists(keys, names [][]byte, tags []models.Tags) error
-	DeleteSeriesRange(itr SeriesIterator, min, max int64) error
-	DeleteSeriesRangeWithPredicate(itr SeriesIterator, predicate func(name []byte, tags models.Tags) (int64, int64, bool)) error
+	DeleteSeriesRange(ctx context.Context, itr SeriesIterator, min, max int64) error
+	DeleteSeriesRangeWithPredicate(ctx context.Context, itr SeriesIterator, predicate func(name []byte, tags models.Tags) (int64, int64, bool)) error
 
 	MeasurementsSketches() (estimator.Sketch, estimator.Sketch, error)
 	SeriesSketches() (estimator.Sketch, estimator.Sketch, error)
@@ -68,19 +68,19 @@ type Engine interface {
 	MeasurementFieldSet() *MeasurementFieldSet
 	MeasurementFields(measurement []byte) *MeasurementFields
 	ForEachMeasurementName(fn func(name []byte) error) error
-	DeleteMeasurement(name []byte) error
+	DeleteMeasurement(ctx context.Context, name []byte) error
 
 	HasTagKey(name, key []byte) (bool, error)
 	MeasurementTagKeysByExpr(name []byte, expr influxql.Expr) (map[string]struct{}, error)
 	TagKeyCardinality(name, key []byte) int
 
-	// Statistics will return statistics relevant to this engine.
-	Statistics(tags map[string]string) []models.Statistic
 	LastModified() time.Time
 	DiskSize() int64
 	IsIdle() (bool, string)
 	Free() error
 
+	Reindex() error
+
 	io.WriterTo
 }
 
@@ -92,11 +92,6 @@ type SeriesIDSets interface {
 // EngineFormat represents the format for an engine.
 type EngineFormat int
 
-const (
-	// TSM1Format is the format used by the tsm1 engine.
-	TSM1Format EngineFormat = 2
-)
-
 // NewEngineFunc creates a new engine.
 type NewEngineFunc func(id uint64, i Index, path string, walPath string, sfile *SeriesFile, options EngineOptions) Engine
 
@@ -161,7 +156,6 @@ type EngineOptions struct {
 	EngineVersion string
 	IndexVersion  string
 	ShardID       uint64
-	InmemIndex    interface{} // shared in-memory index
 
 	// Limits the concurrent number of TSM files that can be loaded at once.
 	OpenLimiter limiter.Fixed
@@ -187,13 +181,13 @@ type EngineOptions struct {
 	// nil will allow all combinations to pass.
 	ShardFilter func(database, rp string, id uint64) bool
 
-	Config         Config
-	SeriesIDSets   SeriesIDSets
-	FieldValidator FieldValidator
+	Config       Config
+	SeriesIDSets SeriesIDSets
 
 	OnNewEngine func(Engine)
 
 	FileStoreObserver FileStoreObserver
+	MetricsDisabled   bool
 }
 
 // NewEngineOptions constructs an EngineOptions object with safe default values.
@@ -208,9 +202,6 @@ func NewEngineOptions() EngineOptions {
 	}
 }
 
-// NewInmemIndex returns a new "inmem" index type.
-var NewInmemIndex func(name string, sfile *SeriesFile) (interface{}, error)
-
 type CompactionPlannerCreator func(cfg Config) interface{}
 
 // FileStoreObserver is passed notifications before the file store adds or deletes files. In this way, it can
