@@ -1,14 +1,13 @@
 package tsm1
 
 import (
-	"os"
 	"runtime"
 	"testing"
 	"time"
 
-	"github.com/influxdata/influxdb/logger"
-	"github.com/influxdata/influxdb/query"
+	"github.com/influxdata/influxdb/v2/influxql/query"
 	"github.com/influxdata/influxql"
+	"go.uber.org/zap/zaptest"
 )
 
 func BenchmarkIntegerIterator_Next(b *testing.B) {
@@ -71,7 +70,7 @@ func TestFinalizerIterator(t *testing.T) {
 		step3 = make(chan struct{})
 	)
 
-	l := logger.New(os.Stderr)
+	l := zaptest.NewLogger(t)
 	done := make(chan struct{})
 	func() {
 		itr := &testFinalizerIterator{
@@ -86,17 +85,21 @@ func TestFinalizerIterator(t *testing.T) {
 		newFinalizerIterator(itr, l)
 	}()
 
-	for i := 0; i < 100; i++ {
+	// Thrash the GC here and hope it eventually notices that itr (above) should be
+	// cleaned up, triggering the finalizer.
+	for i := 0; i < 500; i++ {
 		runtime.GC()
 	}
 
-	timer := time.NewTimer(100 * time.Millisecond)
+	// Wait for the finalizer to close the 1st channel.
+	timer := time.NewTimer(1000 * time.Millisecond)
 	select {
 	case <-timer.C:
+		defer close(done)
 		t.Fatal("The finalizer for the iterator did not run")
-		close(done)
 	case <-step1:
-		// The finalizer has successfully run, but should not have completed yet.
+		// The finalizer has successfully started running, but it won't complete
+		// until we close the done channel.
 		timer.Stop()
 	}
 
@@ -123,11 +126,11 @@ func TestFinalizerIterator(t *testing.T) {
 		t.Log("This should never be output")
 	}
 
-	for i := 0; i < 100; i++ {
+	for i := 0; i < 500; i++ {
 		runtime.GC()
 	}
 
-	timer.Reset(100 * time.Millisecond)
+	timer.Reset(1000 * time.Millisecond)
 	select {
 	case <-timer.C:
 		t.Fatal("The second finalizer did not run")
@@ -138,7 +141,7 @@ func TestFinalizerIterator(t *testing.T) {
 	}
 
 	// Wait for step3 to finish where the closed value should be set.
-	timer.Reset(100 * time.Millisecond)
+	timer.Reset(1000 * time.Millisecond)
 	select {
 	case <-timer.C:
 		t.Fatal("The iterator was not finalized")
