@@ -12,9 +12,9 @@ import (
 	"sync"
 	"sync/atomic"
 
-	"github.com/influxdata/influxdb/pkg/bytesutil"
-	"github.com/influxdata/influxdb/pkg/file"
-	"github.com/influxdata/influxdb/tsdb"
+	"github.com/influxdata/influxdb/v2/pkg/bytesutil"
+	"github.com/influxdata/influxdb/v2/pkg/file"
+	"github.com/influxdata/influxdb/v2/tsdb"
 )
 
 // ErrFileInUse is returned when attempting to remove or close a TSM file that is still being used.
@@ -238,6 +238,7 @@ func NewTSMReader(f *os.File, options ...tsmReaderOption) (*TSMReader, error) {
 
 	index, err := t.accessor.init()
 	if err != nil {
+		_ = t.accessor.close()
 		return nil, err
 	}
 
@@ -730,7 +731,7 @@ type indirectIndex struct {
 	// key.
 	offsets []byte
 
-	// minKey, maxKey are the minium and maximum (lexicographically sorted) contained in the
+	// minKey, maxKey are the minimum and maximum (lexicographically sorted) contained in the
 	// file
 	minKey, maxKey []byte
 
