@@ -5,6 +5,9 @@ import (
 	"reflect"
 	"testing"
 	"testing/quick"
+
+	"github.com/google/go-cmp/cmp"
+	"github.com/influxdata/influxdb/v2/internal/testutil"
 )
 
 func Test_StringEncoder_NoValues(t *testing.T) {
@@ -88,9 +91,8 @@ func Test_StringEncoder_Multi_Compressed(t *testing.T) {
 
 func Test_StringEncoder_Quick(t *testing.T) {
 	quick.Check(func(values []string) bool {
-		expected := values
 		if values == nil {
-			expected = []string{}
+			values = []string{}
 		}
 		// Write values to encoder.
 		enc := NewStringEncoder(1024)
@@ -118,8 +120,8 @@ func Test_StringEncoder_Quick(t *testing.T) {
 		}
 
 		// Verify that input and output values match.
-		if !reflect.DeepEqual(expected, got) {
-			t.Fatalf("mismatch:\n\nexp=%#v\n\ngot=%#v\n\n", expected, got)
+		if !reflect.DeepEqual(values, got) {
+			t.Fatalf("mismatch:\n\nexp=%#v\n\ngot=%#v\n\n", values, got)
 		}
 
 		return true
@@ -175,3 +177,49 @@ func Test_StringDecoder_CorruptSetBytes(t *testing.T) {
 		}
 	}
 }
+
+func BenchmarkStringDecoder_DecodeAll(b *testing.B) {
+	benchmarks := []struct {
+		n int
+		w int
+	}{
+		{1, 10},
+		{55, 10},
+		{550, 10},
+		{1000, 10},
+	}
+	for _, bm := range benchmarks {
+		s := NewStringEncoder(bm.n)
+		for c := 0; c < bm.n; c++ {
+			s.Write(testutil.MakeSentence(bm.w))
+		}
+		s.Flush()
+		bytes, err := s.Bytes()
+		if err != nil {
+			b.Fatalf("unexpected error: %v", err)
+		}
+
+		b.Run(fmt.Sprintf("%d", bm.n), func(b *testing.B) {
+			b.SetBytes(int64(len(bytes)))
+			b.ReportAllocs()
+
+			dst := make([]string, bm.n)
+			for i := 0; i < b.N; i++ {
+				var it StringDecoder
+				if err := it.SetBytes(bytes); err != nil {
+					b.Fatalf("unexpected error creating float decoder: %v", err)
+				}
+
+				i := 0
+				for it.Next() {
+					dst[i] = it.Read()
+					i++
+				}
+
+				if len(dst) != bm.n {
+					b.Fatalf("unexpected length -got/+exp\n%s", cmp.Diff(len(dst), bm.n))
+				}
+			}
+		})
+	}
+}
