@@ -1,6 +1,8 @@
 package tsm1
 
 import (
+	"fmt"
+	"math/rand"
 	"reflect"
 	"testing"
 	"testing/quick"
@@ -555,6 +557,10 @@ func BenchmarkTimeEncoder(b *testing.B) {
 	b.ResetTimer()
 	for i := 0; i < b.N; i++ {
 		enc.Bytes()
+		enc.Reset()
+		for i := 0; i < len(x); i++ {
+			enc.Write(x[i])
+		}
 	}
 }
 
@@ -569,10 +575,7 @@ func BenchmarkTimeDecoder_Packed(b *testing.B) {
 
 	b.ResetTimer()
 
-	b.StopTimer()
 	var dec TimeDecoder
-	b.StartTimer()
-
 	for i := 0; i < b.N; i++ {
 		dec.Init(bytes)
 		for dec.Next() {
@@ -601,3 +604,123 @@ func BenchmarkTimeDecoder_RLE(b *testing.B) {
 		}
 	}
 }
+
+func BenchmarkTimeBatch_DecodeAllUncompressed(b *testing.B) {
+	benchmarks := []int{
+		5,
+		55,
+		555,
+		1000,
+	}
+
+	values := []int64{
+		-2352281900722994752, 1438442655375607923, -4110452567888190110,
+		-1221292455668011702, -1941700286034261841, -2836753127140407751,
+		1432686216250034552, 3663244026151507025, -3068113732684750258,
+		-1949953187327444488, 3713374280993588804, 3226153669854871355,
+		-2093273755080502606, 1006087192578600616, -2272122301622271655,
+		2533238229511593671, -4450454445568858273, 2647789901083530435,
+		2761419461769776844, -1324397441074946198, -680758138988210958,
+		94468846694902125, -2394093124890745254, -2682139311758778198,
+	}
+
+	for _, size := range benchmarks {
+		seededRand := rand.New(rand.NewSource(int64(size * 1e3)))
+
+		enc := NewTimeEncoder(size)
+		for i := 0; i < size; i++ {
+			enc.Write(values[seededRand.Int()%len(values)])
+		}
+		bytes, _ := enc.Bytes()
+
+		b.Run(fmt.Sprintf("%d", size), func(b *testing.B) {
+			b.SetBytes(int64(len(bytes)))
+			b.ReportAllocs()
+
+			dst := make([]int64, size)
+			for i := 0; i < b.N; i++ {
+				var dec TimeDecoder
+				dec.Init(bytes)
+				var n int
+				for dec.Next() {
+					dst[n] = dec.Read()
+					n++
+				}
+			}
+		})
+	}
+}
+
+func BenchmarkTimeBatch_DecodeAllPackedSimple(b *testing.B) {
+	benchmarks := []struct {
+		n int
+	}{
+		{5},
+		{55},
+		{555},
+		{1000},
+	}
+	for _, bm := range benchmarks {
+		seededRand := rand.New(rand.NewSource(int64(bm.n * 1e3)))
+
+		enc := NewTimeEncoder(bm.n)
+		for i := 0; i < bm.n; i++ {
+			// Small amount of randomness prevents RLE from being used
+			enc.Write(int64(i*1000) + int64(seededRand.Intn(10)))
+		}
+		bytes, _ := enc.Bytes()
+
+		b.Run(fmt.Sprintf("%d", bm.n), func(b *testing.B) {
+			b.SetBytes(int64(len(bytes)))
+			b.ReportAllocs()
+
+			dst := make([]int64, bm.n)
+			for i := 0; i < b.N; i++ {
+				var dec TimeDecoder
+				dec.Init(bytes)
+				var n int
+				for dec.Next() {
+					dst[n] = dec.Read()
+					n++
+				}
+			}
+		})
+	}
+}
+
+func BenchmarkTimeBatch_DecodeAllRLE(b *testing.B) {
+	benchmarks := []struct {
+		n     int
+		delta int64
+	}{
+		{5, 10},
+		{55, 10},
+		{555, 10},
+		{1000, 10},
+	}
+	for _, bm := range benchmarks {
+		enc := NewTimeEncoder(bm.n)
+		acc := int64(0)
+		for i := 0; i < bm.n; i++ {
+			enc.Write(acc)
+			acc += bm.delta
+		}
+		bytes, _ := enc.Bytes()
+
+		b.Run(fmt.Sprintf("%d_delta_%d", bm.n, bm.delta), func(b *testing.B) {
+			b.SetBytes(int64(len(bytes)))
+			b.ReportAllocs()
+
+			dst := make([]int64, bm.n)
+			for i := 0; i < b.N; i++ {
+				var dec TimeDecoder
+				dec.Init(bytes)
+				var n int
+				for dec.Next() {
+					dst[n] = dec.Read()
+					n++
+				}
+			}
+		})
+	}
+}
