@@ -7,14 +7,17 @@ import (
 	"errors"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"math"
 	"os"
 	"path/filepath"
 	"strings"
 	"sync"
+
+	"github.com/influxdata/influxdb/v2/pkg/file"
+	"github.com/influxdata/influxdb/v2/tsdb"
 )
 
+const TombstoneFileExtension = "tombstone"
 const (
 	headerSize = 4
 	v2header   = 0x1502
@@ -34,15 +37,14 @@ type Tombstoner struct {
 
 	FilterFn func(k []byte) bool
 
+	// Tombstones that have been written but not flushed to disk yet.
+	tombstones []Tombstone
 	// cache of the stats for this tombstone
-	fileStats []FileStat
+	tombstoneStats TombstoneStat
 	// indicates that the stats may be out of sync with what is on disk and they
 	// should be refreshed.
 	statsLoaded bool
 
-	// Tombstones that have been written but not flushed to disk yet.
-	tombstones []Tombstone
-
 	// These are references used for pending writes that have not been committed.  If
 	// these are nil, then no pending writes are in progress.
 	gz                *gzip.Writer
@@ -50,6 +52,18 @@ type Tombstoner struct {
 	pendingFile       *os.File
 	tmp               [8]byte
 	lastAppliedOffset int64
+
+	// Optional observer for when tombstone files are written.
+	obs tsdb.FileStoreObserver
+}
+
+// NewTombstoner constructs a Tombstoner for the given path. FilterFn can be nil.
+func NewTombstoner(path string, filterFn func(k []byte) bool) *Tombstoner {
+	return &Tombstoner{
+		Path:     path,
+		FilterFn: filterFn,
+		obs:      noFileStoreObserver{},
+	}
 }
 
 // Tombstone represents an individual deletion.
@@ -62,6 +76,11 @@ type Tombstone struct {
 	Min, Max int64
 }
 
+// WithObserver sets a FileStoreObserver for when the tombstone file is written.
+func (t *Tombstoner) WithObserver(obs tsdb.FileStoreObserver) {
+	t.obs = obs
+}
+
 // Add adds the all keys, across all timestamps, to the tombstone.
 func (t *Tombstoner) Add(keys [][]byte) error {
 	return t.AddRange(keys, math.MinInt64, math.MaxInt64)
@@ -162,43 +181,52 @@ func (t *Tombstoner) Delete() error {
 
 // HasTombstones return true if there are any tombstone entries recorded.
 func (t *Tombstoner) HasTombstones() bool {
-	files := t.TombstoneFiles()
+	stats := t.TombstoneStats()
+	if !stats.TombstoneExists {
+		return false
+	}
+	if stats.Size > 0 {
+		return true
+	}
+
 	t.mu.RLock()
 	n := len(t.tombstones)
 	t.mu.RUnlock()
 
-	return len(files) > 0 && files[0].Size > 0 || n > 0
+	return n > 0
 }
 
 // TombstoneFiles returns any tombstone files associated with Tombstoner's TSM file.
-func (t *Tombstoner) TombstoneFiles() []FileStat {
+func (t *Tombstoner) TombstoneStats() TombstoneStat {
 	t.mu.RLock()
 	if t.statsLoaded {
-		stats := t.fileStats
+		stats := t.tombstoneStats
 		t.mu.RUnlock()
 		return stats
 	}
 	t.mu.RUnlock()
 
 	stat, err := os.Stat(t.tombstonePath())
-	if os.IsNotExist(err) || err != nil {
+	if err != nil {
 		t.mu.Lock()
 		// The file doesn't exist so record that we tried to load it so
 		// we don't continue to keep trying.  This is the common case.
 		t.statsLoaded = os.IsNotExist(err)
-		t.fileStats = t.fileStats[:0]
+		t.tombstoneStats.TombstoneExists = false
+		stats := t.tombstoneStats
 		t.mu.Unlock()
-		return nil
+		return stats
 	}
 
 	t.mu.Lock()
-	t.fileStats = append(t.fileStats[:0], FileStat{
-		Path:         t.tombstonePath(),
-		LastModified: stat.ModTime().UnixNano(),
-		Size:         uint32(stat.Size()),
-	})
+	t.tombstoneStats = TombstoneStat{
+		TombstoneExists: true,
+		Path:            t.tombstonePath(),
+		LastModified:    stat.ModTime().UnixNano(),
+		Size:            uint32(stat.Size()),
+	}
 	t.statsLoaded = true
-	stats := t.fileStats
+	stats := t.tombstoneStats
 	t.mu.Unlock()
 
 	return stats
@@ -241,7 +269,7 @@ func (t *Tombstoner) Walk(fn func(t Tombstone) error) error {
 }
 
 func (t *Tombstoner) writeTombstoneV3(tombstones []Tombstone) error {
-	tmp, err := ioutil.TempFile(filepath.Dir(t.Path), "tombstone")
+	tmp, err := os.CreateTemp(filepath.Dir(t.Path), TombstoneFileExtension)
 	if err != nil {
 		return err
 	}
@@ -363,11 +391,15 @@ func (t *Tombstoner) commit() error {
 	tmpFilename := t.pendingFile.Name()
 	t.pendingFile.Close()
 
-	if err := renameFile(tmpFilename, t.tombstonePath()); err != nil {
+	if err := t.obs.FileFinishing(tmpFilename); err != nil {
 		return err
 	}
 
-	if err := syncDir(filepath.Dir(t.tombstonePath())); err != nil {
+	if err := file.RenameFile(tmpFilename, t.tombstonePath()); err != nil {
+		return err
+	}
+
+	if err := file.SyncDir(filepath.Dir(t.tombstonePath())); err != nil {
 		return err
 	}
 
@@ -604,7 +636,7 @@ func (t *Tombstoner) readTombstoneV4(f *os.File, fn func(t Tombstone) error) err
 				}
 
 				keyLen := int(binary.BigEndian.Uint32(b[:4]))
-				if keyLen > len(b)+16 {
+				if keyLen+16 > len(b) {
 					b = make([]byte, keyLen+16)
 				}
 
@@ -662,7 +694,7 @@ func (t *Tombstoner) readTombstoneV4(f *os.File, fn func(t Tombstone) error) err
 }
 
 func (t *Tombstoner) tombstonePath() string {
-	if strings.HasSuffix(t.Path, "tombstone") {
+	if strings.HasSuffix(t.Path, TombstoneFileExtension) {
 		return t.Path
 	}
 
@@ -676,7 +708,7 @@ func (t *Tombstoner) tombstonePath() string {
 	}
 
 	// Append the "tombstone" suffix to create a 0000001.tombstone file
-	return filepath.Join(filepath.Dir(t.Path), filename+".tombstone")
+	return filepath.Join(filepath.Dir(t.Path), filename+"."+TombstoneFileExtension)
 }
 
 func (t *Tombstoner) writeTombstone(dst io.Writer, ts Tombstone) error {
