@@ -9,20 +9,24 @@ import (
 	"sort"
 	"sync"
 
-	"github.com/influxdata/influxdb/models"
-	"github.com/influxdata/influxdb/pkg/bytesutil"
-	"github.com/influxdata/influxdb/pkg/estimator"
-	"github.com/influxdata/influxdb/query"
+	"github.com/influxdata/influxdb/v2"
+	"github.com/influxdata/influxdb/v2/influxql/query"
+	"github.com/influxdata/influxdb/v2/models"
+	"github.com/influxdata/influxdb/v2/pkg/bytesutil"
+	"github.com/influxdata/influxdb/v2/pkg/estimator"
+	"github.com/influxdata/influxdb/v2/pkg/slices"
 	"github.com/influxdata/influxql"
 	"go.uber.org/zap"
 )
 
 // Available index types.
 const (
-	InmemIndexName = "inmem"
-	TSI1IndexName  = "tsi1"
+	TSI1IndexName = "tsi1"
 )
 
+// ErrIndexClosing can be returned to from an Index method if the index is currently closing.
+var ErrIndexClosing = errors.New("index is closing")
+
 type Index interface {
 	Open() error
 	Close() error
@@ -34,14 +38,10 @@ type Index interface {
 	DropMeasurement(name []byte) error
 	ForEachMeasurementName(fn func(name []byte) error) error
 
-	InitializeSeries(keys, names [][]byte, tags []models.Tags) error
 	CreateSeriesIfNotExists(key, name []byte, tags models.Tags) error
 	CreateSeriesListIfNotExists(keys, names [][]byte, tags []models.Tags) error
 	DropSeries(seriesID uint64, key []byte, cascade bool) error
-	DropMeasurementIfSeriesNotExist(name []byte) error
-
-	// Used to clean up series in inmem index that were dropped with a shard.
-	DropSeriesGlobal(key []byte) error
+	DropMeasurementIfSeriesNotExist(name []byte) (bool, error)
 
 	MeasurementsSketches() (estimator.Sketch, estimator.Sketch, error)
 	SeriesN() int64
@@ -73,15 +73,10 @@ type Index interface {
 	// Bytes estimates the memory footprint of this Index, in bytes.
 	Bytes() int
 
-	// To be removed w/ tsi1.
-	SetFieldName(measurement []byte, name string)
-
 	Type() string
-	// Returns a unique reference ID to the index instance.
-	// For inmem, returns a reference to the backing Index, not ShardIndex.
-	UniqueReferenceID() uintptr
 
-	Rebuild()
+	// Returns a unique reference ID to the index instance.
+	UniqueReferenceID() uintptr
 }
 
 // SeriesElem represents a generic series element.
@@ -131,8 +126,10 @@ func (itr *seriesIteratorAdapter) Next() (SeriesElem, error) {
 		}
 
 		name, tags := ParseSeriesKey(key)
+		if len(name) == 0 {
+			continue
+		}
 		deleted := itr.sfile.IsDeleted(elem.SeriesID)
-
 		return &seriesElemAdapter{
 			name:    name,
 			tags:    tags,
@@ -154,6 +151,50 @@ func (e *seriesElemAdapter) Tags() models.Tags   { return e.tags }
 func (e *seriesElemAdapter) Deleted() bool       { return e.deleted }
 func (e *seriesElemAdapter) Expr() influxql.Expr { return e.expr }
 
+var _ SeriesIDIterator = (*PredicateSeriesIDIterator)(nil)
+
+type PredicateSeriesIDIterator struct {
+	itr   SeriesIDIterator
+	sfile *SeriesFile
+	pred  influxdb.Predicate
+}
+
+func NewPredicateSeriesIDIterator(itr SeriesIDIterator, sfile *SeriesFile, pred influxdb.Predicate) SeriesIDIterator {
+	if pred == nil {
+		return itr
+	}
+	return &PredicateSeriesIDIterator{
+		itr:   itr,
+		sfile: sfile,
+		pred:  pred,
+	}
+}
+
+func (itr *PredicateSeriesIDIterator) Close() error { return itr.itr.Close() }
+
+func (itr *PredicateSeriesIDIterator) Next() (SeriesIDElem, error) {
+	for {
+		elem, err := itr.itr.Next()
+		if elem.SeriesID == 0 || err != nil {
+			return elem, err
+		}
+
+		// Skip if this key has been tombstoned.
+		seriesKey := itr.sfile.SeriesKey(elem.SeriesID)
+		if len(seriesKey) == 0 {
+			continue
+		}
+
+		name, tags := ParseSeriesKey(seriesKey)
+		tags = append(models.Tags{{Key: models.MeasurementTagKeyBytes, Value: name}}, tags...)
+		key := models.MakeKey(name, tags)
+		if !itr.pred.Matches(key) {
+			continue
+		}
+		return elem, nil
+	}
+}
+
 // SeriesIDElem represents a single series and optional expression.
 type SeriesIDElem struct {
 	SeriesID uint64
@@ -173,6 +214,12 @@ type SeriesIDIterator interface {
 	Close() error
 }
 
+// SeriesKeyIterator represents an iterator over a list of SeriesKeys
+type SeriesKeyIterator interface {
+	Next() ([]byte, error)
+	Close() error
+}
+
 // SeriesIDSetIterator represents an iterator that can produce a SeriesIDSet.
 type SeriesIDSetIterator interface {
 	SeriesIDIterator
@@ -346,6 +393,9 @@ func (itr *seriesQueryAdapterIterator) Next() (*query.FloatPoint, error) {
 
 		// Convert to a key.
 		name, tags := ParseSeriesKey(seriesKey)
+		if len(name) == 0 {
+			continue
+		}
 		key := string(models.MakeKey(name, tags))
 
 		// Write auxiliary fields.
@@ -426,7 +476,7 @@ func (itr *seriesIDExprIterator) Next() (SeriesIDElem, error) {
 }
 
 // MergeSeriesIDIterators returns an iterator that merges a set of iterators.
-// Iterators that are first in the list take precendence and a deletion by those
+// Iterators that are first in the list take precedence and a deletion by those
 // early iterators will invalidate elements by later iterators.
 func MergeSeriesIDIterators(itrs ...SeriesIDIterator) SeriesIDIterator {
 	if n := len(itrs); n == 0 {
@@ -519,9 +569,11 @@ func IntersectSeriesIDIterators(itr0, itr1 SeriesIDIterator) SeriesIDIterator {
 
 	// Create series id set, if available.
 	if a := NewSeriesIDSetIterators([]SeriesIDIterator{itr0, itr1}); a != nil {
+		ss := a[0].SeriesIDSet().And(a[1].SeriesIDSet())
+		// `a` holds references to itr0/itr1 so itr0/itr1 should not be closed when `a` is still in use
 		itr0.Close()
 		itr1.Close()
-		return NewSeriesIDSetIterator(a[0].SeriesIDSet().And(a[1].SeriesIDSet()))
+		return NewSeriesIDSetIterator(ss)
 	}
 
 	return &seriesIDIntersectIterator{itrs: [2]SeriesIDIterator{itr0, itr1}}
@@ -608,10 +660,11 @@ func UnionSeriesIDIterators(itr0, itr1 SeriesIDIterator) SeriesIDIterator {
 
 	// Create series id set, if available.
 	if a := NewSeriesIDSetIterators([]SeriesIDIterator{itr0, itr1}); a != nil {
-		itr0.Close()
-		itr1.Close()
 		ss := NewSeriesIDSet()
 		ss.Merge(a[0].SeriesIDSet(), a[1].SeriesIDSet())
+		// `a` holds references to itr0/itr1 so itr0/itr1 should not be closed when `a` is still in use
+		itr0.Close()
+		itr1.Close()
 		return NewSeriesIDSetIterator(ss)
 	}
 
@@ -695,9 +748,11 @@ func DifferenceSeriesIDIterators(itr0, itr1 SeriesIDIterator) SeriesIDIterator {
 
 	// Create series id set, if available.
 	if a := NewSeriesIDSetIterators([]SeriesIDIterator{itr0, itr1}); a != nil {
+		ss := a[0].SeriesIDSet().AndNot(a[1].SeriesIDSet())
+		// `a` holds references to itr0/itr1 so itr0/itr1 should not be closed when `a` is still in use
 		itr0.Close()
 		itr1.Close()
-		return NewSeriesIDSetIterator(a[0].SeriesIDSet().AndNot(a[1].SeriesIDSet()))
+		return NewSeriesIDSetIterator(ss)
 	}
 
 	return &seriesIDDifferenceIterator{itrs: [2]SeriesIDIterator{itr0, itr1}}
@@ -837,6 +892,9 @@ func (itr *seriesPointIterator) Next() (*query.FloatPoint, error) {
 		}
 
 		name, tags := ParseSeriesKey(itr.keys[0])
+		if len(name) == 0 {
+			continue
+		}
 		itr.keys = itr.keys[1:]
 
 		// TODO(edd): It seems to me like this authorisation check should be
@@ -919,6 +977,11 @@ func (a MeasurementIterators) Close() (err error) {
 	return err
 }
 
+type MeasurementSliceIterator interface {
+	MeasurementIterator
+	UnderlyingSlice() [][]byte
+}
+
 type measurementSliceIterator struct {
 	names [][]byte
 }
@@ -938,8 +1001,39 @@ func (itr *measurementSliceIterator) Next() (name []byte, err error) {
 	return name, nil
 }
 
+func (itr *measurementSliceIterator) UnderlyingSlice() [][]byte {
+	return itr.names
+}
+
+// fileMeasurementSliceIterator is designed to allow a tag value slice
+// iterator to use memory from a memory-mapped file, pinning it
+// with the underlying file iterators
+type fileMeasurementSliceIterator struct {
+	measurementSliceIterator
+	fileIterators MeasurementIterators
+}
+
+func (itr *fileMeasurementSliceIterator) Close() error {
+	e1 := itr.fileIterators.Close()
+	e2 := itr.measurementSliceIterator.Close()
+	if e1 != nil {
+		return e1
+	} else {
+		return e2
+	}
+}
+
+func newFileMeasurementSliceIterator(names [][]byte, itrs MeasurementIterators) *fileMeasurementSliceIterator {
+	return &fileMeasurementSliceIterator{
+		measurementSliceIterator: measurementSliceIterator{
+			names: names,
+		},
+		fileIterators: itrs,
+	}
+}
+
 // MergeMeasurementIterators returns an iterator that merges a set of iterators.
-// Iterators that are first in the list take precendence and a deletion by those
+// Iterators that are first in the list take precedence and a deletion by those
 // early iterators will invalidate elements by later iterators.
 func MergeMeasurementIterators(itrs ...MeasurementIterator) MeasurementIterator {
 	if len(itrs) == 0 {
@@ -1225,16 +1319,6 @@ type IndexSet struct {
 	fieldSets  []*MeasurementFieldSet // field sets for _all_ indexes in this set's DB.
 }
 
-// HasInmemIndex returns true if any in-memory index is in use.
-func (is IndexSet) HasInmemIndex() bool {
-	for _, idx := range is.Indexes {
-		if idx.Type() == InmemIndexName {
-			return true
-		}
-	}
-	return false
-}
-
 // Database returns the database name of the first index.
 func (is IndexSet) Database() string {
 	if len(is.Indexes) == 0 {
@@ -1266,37 +1350,26 @@ func (is IndexSet) HasField(measurement []byte, field string) bool {
 	return false
 }
 
-// DedupeInmemIndexes returns an index set which removes duplicate indexes.
-// Useful because inmem indexes are shared by shards per database.
-func (is IndexSet) DedupeInmemIndexes() IndexSet {
-	other := IndexSet{
-		Indexes:    make([]Index, 0, len(is.Indexes)),
-		SeriesFile: is.SeriesFile,
-		fieldSets:  make([]*MeasurementFieldSet, 0, len(is.Indexes)),
-	}
-
-	uniqueIndexes := make(map[uintptr]Index)
-	for _, idx := range is.Indexes {
-		uniqueIndexes[idx.UniqueReferenceID()] = idx
-	}
-
-	for _, idx := range uniqueIndexes {
-		other.Indexes = append(other.Indexes, idx)
-		other.fieldSets = append(other.fieldSets, idx.FieldSet())
-	}
-
-	return other
-}
-
 // MeasurementNamesByExpr returns a slice of measurement names matching the
 // provided condition. If no condition is provided then all names are returned.
-func (is IndexSet) MeasurementNamesByExpr(auth query.Authorizer, expr influxql.Expr) ([][]byte, error) {
+func (is IndexSet) MeasurementNamesByExpr(auth query.Authorizer, expr influxql.Expr) (_ [][]byte, err error) {
 	release := is.SeriesFile.Retain()
 	defer release()
 
 	// Return filtered list if expression exists.
 	if expr != nil {
-		return is.measurementNamesByExpr(auth, expr)
+		itr, returnErr := is.measurementNamesByExpr(auth, expr)
+		if returnErr != nil {
+			return nil, returnErr
+		} else if itr == nil {
+			return nil, nil
+		}
+		defer func() {
+			if e := itr.Close(); err == nil {
+				err = e
+			}
+		}()
+		return slices.CopyChunkedByteSlices(itr.UnderlyingSlice(), 1000), nil
 	}
 
 	itr, err := is.measurementIterator()
@@ -1305,10 +1378,14 @@ func (is IndexSet) MeasurementNamesByExpr(auth query.Authorizer, expr influxql.E
 	} else if itr == nil {
 		return nil, nil
 	}
-	defer itr.Close()
+	defer func() {
+		if e := itr.Close(); err == nil {
+			err = e
+		}
+	}()
 
-	// Iterate over all measurements if no condition exists.
 	var names [][]byte
+	// Iterate over all measurements if no condition exists.
 	for {
 		e, err := itr.Next()
 		if err != nil {
@@ -1319,14 +1396,14 @@ func (is IndexSet) MeasurementNamesByExpr(auth query.Authorizer, expr influxql.E
 
 		// Determine if there exists at least one authorised series for the
 		// measurement name.
-		if is.measurementAuthorizedSeries(auth, e) {
+		if is.measurementAuthorizedSeries(auth, e, nil) {
 			names = append(names, e)
 		}
 	}
-	return names, nil
+	return slices.CopyChunkedByteSlices(names, 1000), nil
 }
 
-func (is IndexSet) measurementNamesByExpr(auth query.Authorizer, expr influxql.Expr) ([][]byte, error) {
+func (is IndexSet) measurementNamesByExpr(auth query.Authorizer, expr influxql.Expr) (MeasurementSliceIterator, error) {
 	if expr == nil {
 		return nil, nil
 	}
@@ -1366,20 +1443,22 @@ func (is IndexSet) measurementNamesByExpr(auth query.Authorizer, expr influxql.E
 			return is.measurementNamesByTagFilter(auth, e.Op, tag.Val, value, regex)
 
 		case influxql.OR, influxql.AND:
+
 			lhs, err := is.measurementNamesByExpr(auth, e.LHS)
 			if err != nil {
 				return nil, err
 			}
-
 			rhs, err := is.measurementNamesByExpr(auth, e.RHS)
 			if err != nil {
+				lhs.Close()
 				return nil, err
 			}
 
+			mis := MeasurementIterators{lhs, rhs}
 			if e.Op == influxql.OR {
-				return bytesutil.Union(lhs, rhs), nil
+				return newFileMeasurementSliceIterator(bytesutil.Union(lhs.UnderlyingSlice(), rhs.UnderlyingSlice()), mis), nil
 			}
-			return bytesutil.Intersect(lhs, rhs), nil
+			return newFileMeasurementSliceIterator(bytesutil.Intersect(lhs.UnderlyingSlice(), rhs.UnderlyingSlice()), mis), nil
 
 		default:
 			return nil, fmt.Errorf("invalid tag comparison operator")
@@ -1388,24 +1467,24 @@ func (is IndexSet) measurementNamesByExpr(auth query.Authorizer, expr influxql.E
 	case *influxql.ParenExpr:
 		return is.measurementNamesByExpr(auth, e.Expr)
 	default:
-		return nil, fmt.Errorf("%#v", expr)
+		return nil, fmt.Errorf("invalid measurement expression %#v", expr)
 	}
 }
 
 // measurementNamesByNameFilter returns matching measurement names in sorted order.
-func (is IndexSet) measurementNamesByNameFilter(auth query.Authorizer, op influxql.Token, val string, regex *regexp.Regexp) ([][]byte, error) {
+func (is IndexSet) measurementNamesByNameFilter(auth query.Authorizer, op influxql.Token, val string, regex *regexp.Regexp) (MeasurementSliceIterator, error) {
 	itr, err := is.measurementIterator()
 	if err != nil {
 		return nil, err
 	} else if itr == nil {
 		return nil, nil
 	}
-	defer itr.Close()
 
 	var names [][]byte
 	for {
 		e, err := itr.Next()
 		if err != nil {
+			itr.Close()
 			return nil, err
 		} else if e == nil {
 			break
@@ -1423,16 +1502,139 @@ func (is IndexSet) measurementNamesByNameFilter(auth query.Authorizer, op influx
 			matched = !regex.Match(e)
 		}
 
-		if matched && is.measurementAuthorizedSeries(auth, e) {
+		if matched && is.measurementAuthorizedSeries(auth, e, nil) {
 			names = append(names, e)
 		}
 	}
 	bytesutil.Sort(names)
-	return names, nil
+	return newFileMeasurementSliceIterator(names, MeasurementIterators{itr}), nil
 }
 
-func (is IndexSet) measurementNamesByTagFilter(auth query.Authorizer, op influxql.Token, key, val string, regex *regexp.Regexp) ([][]byte, error) {
+// MeasurementNamesByPredicate returns a slice of measurement names matching the
+// provided condition. If no condition is provided then all names are returned.
+// This behaves differently from MeasurementNamesByExpr because it will
+// return measurements using flux predicates.
+func (is IndexSet) MeasurementNamesByPredicate(auth query.Authorizer, expr influxql.Expr) (_ [][]byte, err error) {
+	release := is.SeriesFile.Retain()
+	defer release()
+
+	// Return filtered list if expression exists.
+	if expr != nil {
+		itr, returnErr := is.measurementNamesByPredicate(auth, expr)
+		if returnErr != nil {
+			return nil, returnErr
+		}
+		if itr != nil {
+			defer func() {
+				if e := itr.Close(); err == nil {
+					err = e
+				}
+			}()
+		}
+		return slices.CopyChunkedByteSlices(itr.UnderlyingSlice(), 1000), nil
+	}
+
+	itr, err := is.measurementIterator()
+	if err != nil {
+		return nil, err
+	} else if itr == nil {
+		return nil, nil
+	}
+	defer func() {
+		if e := itr.Close(); err == nil {
+			err = e
+		}
+	}()
+
 	var names [][]byte
+	// Iterate over all measurements if no condition exists.
+	for {
+		e, err := itr.Next()
+		if err != nil {
+			return nil, err
+		} else if e == nil {
+			break
+		}
+
+		// Determine if there exists at least one authorised series for the
+		// measurement name.
+		if is.measurementAuthorizedSeries(auth, e, nil) {
+			names = append(names, e)
+		}
+	}
+	return slices.CopyChunkedByteSlices(names, 1000), nil
+}
+
+func (is IndexSet) measurementNamesByPredicate(auth query.Authorizer, expr influxql.Expr) (MeasurementSliceIterator, error) {
+	if expr == nil {
+		return nil, nil
+	}
+
+	switch e := expr.(type) {
+	case *influxql.BinaryExpr:
+		switch e.Op {
+		case influxql.EQ, influxql.NEQ, influxql.EQREGEX, influxql.NEQREGEX:
+			tag, ok := e.LHS.(*influxql.VarRef)
+			if !ok {
+				return nil, fmt.Errorf("left side of '%s' must be a tag key", e.Op.String())
+			}
+
+			// Retrieve value or regex expression from RHS.
+			var value string
+			var regex *regexp.Regexp
+			if influxql.IsRegexOp(e.Op) {
+				re, ok := e.RHS.(*influxql.RegexLiteral)
+				if !ok {
+					return nil, fmt.Errorf("right side of '%s' must be a regular expression", e.Op.String())
+				}
+				regex = re.Val
+			} else {
+				s, ok := e.RHS.(*influxql.StringLiteral)
+				if !ok {
+					return nil, fmt.Errorf("right side of '%s' must be a tag value string", e.Op.String())
+				}
+				value = s.Val
+			}
+
+			// Match on name, if specified.
+			if tag.Val == "_name" {
+				return is.measurementNamesByNameFilter(auth, e.Op, value, regex)
+			} else if influxql.IsSystemName(tag.Val) {
+				return nil, nil
+			}
+			return is.measurementNamesByTagPredicate(auth, e.Op, tag.Val, value, regex)
+
+		case influxql.OR, influxql.AND:
+			lhs, err := is.measurementNamesByPredicate(auth, e.LHS)
+			if err != nil {
+				return nil, err
+			}
+			rhs, err := is.measurementNamesByPredicate(auth, e.RHS)
+			if err != nil {
+				lhs.Close()
+				return nil, err
+			}
+			mis := MeasurementIterators{lhs, rhs}
+
+			if e.Op == influxql.OR {
+				return newFileMeasurementSliceIterator(bytesutil.Union(lhs.UnderlyingSlice(), rhs.UnderlyingSlice()), mis), nil
+			}
+			return newFileMeasurementSliceIterator(bytesutil.Intersect(lhs.UnderlyingSlice(), rhs.UnderlyingSlice()), mis), nil
+
+		default:
+			return nil, fmt.Errorf("invalid tag comparison operator")
+		}
+
+	case *influxql.ParenExpr:
+		return is.measurementNamesByPredicate(auth, e.Expr)
+	default:
+		return nil, fmt.Errorf("%#v", expr)
+	}
+}
+
+func (is IndexSet) measurementNamesByTagFilter(auth query.Authorizer, op influxql.Token, key, val string, regex *regexp.Regexp) (MeasurementSliceIterator, error) {
+	var names [][]byte
+	failed := true
 
 	mitr, err := is.measurementIterator()
 	if err != nil {
@@ -1440,7 +1642,11 @@ func (is IndexSet) measurementNamesByTagFilter(auth query.Authorizer, op influxq
 	} else if mitr == nil {
 		return nil, nil
 	}
-	defer mitr.Close()
+	defer func() {
+		if failed {
+			mitr.Close()
+		}
+	}()
 
 	// valEqual determines if the provided []byte is equal to the tag value
 	// to be filtered on.
@@ -1539,7 +1745,7 @@ func (is IndexSet) measurementNamesByTagFilter(auth query.Authorizer, op influxq
 		// an authorized series belonging to the measurement must be located.
 		// Then, the measurement can be added iff !tagMatch && authorized.
 		if (op == influxql.NEQ || op == influxql.NEQREGEX) && !tagMatch {
-			authorized = is.measurementAuthorizedSeries(auth, me)
+			authorized = is.measurementAuthorizedSeries(auth, me, nil)
 		}
 
 		// tags match | operation is EQ | measurement matches
@@ -1555,16 +1761,90 @@ func (is IndexSet) measurementNamesByTagFilter(auth query.Authorizer, op influxq
 	}
 
 	bytesutil.Sort(names)
-	return names, nil
+	failed = false
+	return newFileMeasurementSliceIterator(names, MeasurementIterators{mitr}), nil
+}
+
+func (is IndexSet) measurementNamesByTagPredicate(auth query.Authorizer, op influxql.Token, key, val string, regex *regexp.Regexp) (MeasurementSliceIterator, error) {
+	var names [][]byte
+	failed := true
+
+	mitr, err := is.measurementIterator()
+	if err != nil {
+		return nil, err
+	} else if mitr == nil {
+		return nil, nil
+	}
+	defer func() {
+		if failed {
+			mitr.Close()
+		}
+	}()
+
+	var checkMeasurement func(auth query.Authorizer, me []byte) (bool, error)
+	switch op {
+	case influxql.EQ:
+		checkMeasurement = func(auth query.Authorizer, me []byte) (bool, error) {
+			return is.measurementHasTagValue(auth, me, []byte(key), []byte(val))
+		}
+	case influxql.NEQ:
+		checkMeasurement = func(auth query.Authorizer, me []byte) (bool, error) {
+			// If there is an authorized series in this measurement and that series
+			// does not contain the tag key/value.
+			ok := is.measurementAuthorizedSeries(auth, me, func(tags models.Tags) bool {
+				return tags.GetString(key) == val
+			})
+			return ok, nil
+		}
+	case influxql.EQREGEX:
+		checkMeasurement = func(auth query.Authorizer, me []byte) (bool, error) {
+			return is.measurementHasTagValueRegex(auth, me, []byte(key), regex)
+		}
+	case influxql.NEQREGEX:
+		checkMeasurement = func(auth query.Authorizer, me []byte) (bool, error) {
+			// If there is an authorized series in this measurement and that series
+			// does not contain the tag key/value.
+			ok := is.measurementAuthorizedSeries(auth, me, func(tags models.Tags) bool {
+				return regex.MatchString(tags.GetString(key))
+			})
+			return ok, nil
+		}
+	default:
+		return nil, fmt.Errorf("unsupported operand: %s", op)
+	}
+
+	for {
+		me, err := mitr.Next()
+		if err != nil {
+			return nil, err
+		} else if me == nil {
+			break
+		}
+
+		ok, err := checkMeasurement(auth, me)
+		if err != nil {
+			return nil, err
+		} else if ok {
+			names = append(names, me)
+		}
+	}
+
+	bytesutil.Sort(names)
+	failed = false
+	return newFileMeasurementSliceIterator(names, MeasurementIterators{mitr}), nil
 }
 
 // measurementAuthorizedSeries determines if the measurement contains a series
 // that is authorized to be read.
-func (is IndexSet) measurementAuthorizedSeries(auth query.Authorizer, name []byte) bool {
-	if query.AuthorizerIsOpen(auth) {
+func (is IndexSet) measurementAuthorizedSeries(auth query.Authorizer, name []byte, exclude func(tags models.Tags) bool) bool {
+	if query.AuthorizerIsOpen(auth) && exclude == nil {
 		return true
 	}
 
+	if auth == nil {
+		auth = query.OpenAuthorizer
+	}
+
 	sitr, err := is.measurementSeriesIDIterator(name)
 	if err != nil || sitr == nil {
 		return false
@@ -1584,11 +1864,151 @@ func (is IndexSet) measurementAuthorizedSeries(auth query.Authorizer, name []byt
 
 		name, tags := is.SeriesFile.Series(series.SeriesID)
 		if auth.AuthorizeSeriesRead(is.Database(), name, tags) {
+			if exclude != nil && exclude(tags) {
+				continue
+			}
 			return true
 		}
 	}
 }
 
+func (is IndexSet) measurementHasTagValue(auth query.Authorizer, me, key, value []byte) (bool, error) {
+	if len(value) == 0 {
+		return is.measurementHasEmptyTagValue(auth, me, key)
+	}
+
+	hasTagValue, err := is.HasTagValue(me, key, value)
+	if err != nil || !hasTagValue {
+		return false, err
+	}
+
+	// If the authorizer is open, return true.
+	if query.AuthorizerIsOpen(auth) {
+		return true, nil
+	}
+
+	// When an authorizer is present, the measurement should be
+	// included only if one of it's series is authorized.
+	sitr, err := is.tagValueSeriesIDIterator(me, key, value)
+	if err != nil || sitr == nil {
+		return false, err
+	}
+	defer sitr.Close()
+	sitr = FilterUndeletedSeriesIDIterator(is.SeriesFile, sitr)
+
+	// Locate a series with this matching tag value that's authorized.
+	for {
+		se, err := sitr.Next()
+		if err != nil || se.SeriesID == 0 {
+			return false, err
+		}
+
+		name, tags := is.SeriesFile.Series(se.SeriesID)
+		if auth.AuthorizeSeriesRead(is.Database(), name, tags) {
+			return true, nil
+		}
+	}
+}
+
+func (is IndexSet) measurementHasEmptyTagValue(auth query.Authorizer, me, key []byte) (bool, error) {
+	// Any series that does not have a tag key
+	// has an empty tag value for that key.
+	// Iterate through all of the series to find one
+	// series that does not have the tag key.
+	sitr, err := is.measurementSeriesIDIterator(me)
+	if err != nil || sitr == nil {
+		return false, err
+	}
+	defer sitr.Close()
+	sitr = FilterUndeletedSeriesIDIterator(is.SeriesFile, sitr)
+
+	for {
+		series, err := sitr.Next()
+		if err != nil || series.SeriesID == 0 {
+			return false, err
+		}
+
+		name, tags := is.SeriesFile.Series(series.SeriesID)
+		if len(tags.Get(key)) > 0 {
+			// The tag key exists in this series. We need
+			// at least one series that does not have the tag
+			// keys.
+			continue
+		}
+
+		// Verify that we can see this series.
+		if query.AuthorizerIsOpen(auth) {
+			return true, nil
+		} else if auth.AuthorizeSeriesRead(is.Database(), name, tags) {
+			return true, nil
+		}
+	}
+}
+
+func (is IndexSet) measurementHasTagValueRegex(auth query.Authorizer, me, key []byte, value *regexp.Regexp) (bool, error) {
+	// If the regex matches the empty string, do a special check to see
+	// if we have an empty tag value.
+	if matchEmpty := value.MatchString(""); matchEmpty {
+		if ok, err := is.measurementHasEmptyTagValue(auth, me, key); err != nil {
+			return false, err
+		} else if ok {
+			return true, nil
+		}
+	}
+
+	// Iterate over the tag values and find one that matches the value.
+	vitr, err := is.tagValueIterator(me, key)
+	if err != nil || vitr == nil {
+		return false, err
+	}
+	defer vitr.Close()
+
+	for {
+		ve, err := vitr.Next()
+		if err != nil || ve == nil {
+			return false, err
+		}
+
+		if !value.Match(ve) {
+			// The regex does not match this tag value.
+			continue
+		}
+
+		// If the authorizer is open, then we have found a suitable tag value.
+		if query.AuthorizerIsOpen(auth) {
+			return true, nil
+		}
+
+		// When an authorizer is present, the measurement should only be included
+		// if one of the series is authorized.
+		if authorized, err := func() (bool, error) {
+			sitr, err := is.tagValueSeriesIDIterator(me, key, ve)
+			if err != nil || sitr == nil {
+				return false, err
+			}
+			defer sitr.Close()
+			sitr = FilterUndeletedSeriesIDIterator(is.SeriesFile, sitr)
+
+			// Locate an authorized series.
+			for {
+				se, err := sitr.Next()
+				if err != nil || se.SeriesID == 0 {
+					return false, err
+				}
+
+				name, tags := is.SeriesFile.Series(se.SeriesID)
+				if auth.AuthorizeSeriesRead(is.Database(), name, tags) {
+					return true, nil
+				}
+			}
+		}(); err != nil {
+			return false, err
+		} else if authorized {
+			return true, nil
+		}
+	}
+}
+
 // HasTagKey returns true if the tag key exists in any index for the provided
 // measurement.
 func (is IndexSet) HasTagKey(name, key []byte) (bool, error) {
@@ -1687,7 +2107,7 @@ func (is IndexSet) tagValueIterator(name, key []byte) (TagValueIterator, error)
 // TagKeyHasAuthorizedSeries determines if there exists an authorized series for
 // the provided measurement name and tag key.
 func (is IndexSet) TagKeyHasAuthorizedSeries(auth query.Authorizer, name, tagKey []byte) (bool, error) {
-	if !is.HasInmemIndex() && query.AuthorizerIsOpen(auth) {
+	if query.AuthorizerIsOpen(auth) {
 		return true, nil
 	}
 
@@ -1890,6 +2310,96 @@ func (is IndexSet) measurementSeriesByExprIterator(name []byte, expr influxql.Ex
 	return FilterUndeletedSeriesIDIterator(is.SeriesFile, itr), nil
 }
 
+type measurementSeriesKeyByExprIterator struct {
+	ids      SeriesIDIterator
+	is       IndexSet
+	auth     query.Authorizer
+	once     sync.Once
+	releaser func()
+}
+
+func (itr *measurementSeriesKeyByExprIterator) Next() ([]byte, error) {
+	if itr == nil {
+		return nil, nil
+	}
+	for {
+		e, err := itr.ids.Next()
+		if err != nil {
+			return nil, err
+		} else if e.SeriesID == 0 {
+			return nil, nil
+		}
+
+		seriesKey := itr.is.SeriesFile.SeriesKey(e.SeriesID)
+		if len(seriesKey) == 0 {
+			continue
+		}
+
+		name, tags := ParseSeriesKey(seriesKey)
+		if len(name) == 0 {
+			continue
+		}
+
+		// Check leftover filters. All fields that might be filtered default to zero values
+		if e.Expr != nil {
+			if v, ok := e.Expr.(*influxql.BooleanLiteral); ok {
+				if !v.Val {
+					continue
+				}
+			} else {
+				values := make(map[string]interface{}, len(tags))
+				for _, t := range tags {
+					values[string(t.Key)] = string(t.Value)
+				}
+				if !influxql.EvalBool(e.Expr, values) {
+					continue
+				}
+			}
+		}
+
+		if itr.auth != nil && !itr.auth.AuthorizeSeriesRead(itr.is.Database(), name, tags) {
+			continue
+		}
+
+		out := models.MakeKey(name, tags)
+		// ensure nil is only returned when we are done (or for errors)
+		if out == nil {
+			out = []byte{}
+		}
+		return out, nil
+	}
+}
+
+func (itr *measurementSeriesKeyByExprIterator) Close() error {
+	if itr == nil {
+		return nil
+	}
+	itr.once.Do(itr.releaser)
+	return itr.ids.Close()
+}
+
+// MeasurementSeriesKeyByExprIterator iterates through series, filtered by an expression on the tags.
+// Any non-tag expressions will be filtered as if the field had the zero value.
+func (is IndexSet) MeasurementSeriesKeyByExprIterator(name []byte, expr influxql.Expr, auth query.Authorizer) (SeriesKeyIterator, error) {
+	release := is.SeriesFile.Retain()
+	// Create iterator for all matching series.
+	ids, err := is.measurementSeriesByExprIterator(name, expr)
+	if err != nil {
+		release()
+		return nil, err
+	}
+	if ids == nil {
+		release()
+		return nil, nil
+	}
+	return &measurementSeriesKeyByExprIterator{
+		ids:      ids,
+		releaser: release,
+		auth:     auth,
+		is:       is,
+	}, nil
+}
+
 // MeasurementSeriesKeysByExpr returns a list of series keys matching expr.
 func (is IndexSet) MeasurementSeriesKeysByExpr(name []byte, expr influxql.Expr) ([][]byte, error) {
 	release := is.SeriesFile.Retain()
@@ -1930,6 +2440,11 @@ func (is IndexSet) MeasurementSeriesKeysByExpr(name []byte, expr influxql.Expr)
 		}
 
 		name, tags := ParseSeriesKey(seriesKey)
+		// An invalid series key of 0 length should have been caught by the
+		// above check, but for extra safety we can pass over it here too.
+		if len(name) == 0 {
+			continue
+		}
 		keys = append(keys, models.MakeKey(name, tags))
 	}
 
@@ -2200,8 +2715,9 @@ func (is IndexSet) matchTagValueEqualEmptySeriesIDIterator(name, key []byte, val
 				itr, err := is.tagValueSeriesIDIterator(name, key, e)
 				if err != nil {
 					return err
+				} else if itr != nil {
+					itrs = append(itrs, itr)
 				}
-				itrs = append(itrs, itr)
 			}
 		}
 		return nil
@@ -2243,8 +2759,9 @@ func (is IndexSet) matchTagValueEqualNotEmptySeriesIDIterator(name, key []byte,
 			if err != nil {
 				SeriesIDIterators(itrs).Close()
 				return nil, err
+			} else if itr != nil {
+				itrs = append(itrs, itr)
 			}
-			itrs = append(itrs, itr)
 		}
 	}
 	return MergeSeriesIDIterators(itrs...), nil
@@ -2274,8 +2791,9 @@ func (is IndexSet) matchTagValueNotEqualEmptySeriesIDIterator(name, key []byte,
 			if err != nil {
 				SeriesIDIterators(itrs).Close()
 				return nil, err
+			} else if itr != nil {
+				itrs = append(itrs, itr)
 			}
-			itrs = append(itrs, itr)
 		}
 	}
 	return MergeSeriesIDIterators(itrs...), nil
@@ -2304,8 +2822,9 @@ func (is IndexSet) matchTagValueNotEqualNotEmptySeriesIDIterator(name, key []byt
 			if err != nil {
 				SeriesIDIterators(itrs).Close()
 				return nil, err
+			} else if itr != nil {
+				itrs = append(itrs, itr)
 			}
-			itrs = append(itrs, itr)
 		}
 	}
 
@@ -2317,28 +2836,40 @@ func (is IndexSet) matchTagValueNotEqualNotEmptySeriesIDIterator(name, key []byt
 	return DifferenceSeriesIDIterators(mitr, MergeSeriesIDIterators(itrs...)), nil
 }
 
-// TagValuesByKeyAndExpr retrieves tag values for the provided tag keys.
+// tagValuesByKeyAndExpr retrieves tag values for the provided tag keys.
 //
-// TagValuesByKeyAndExpr returns sets of values for each key, indexable by the
+// tagValuesByKeyAndExpr returns sets of values for each key, indexable by the
 // position of the tag key in the keys argument.
 //
 // N.B tagValuesByKeyAndExpr relies on keys being sorted in ascending
 // lexicographic order.
-func (is IndexSet) TagValuesByKeyAndExpr(auth query.Authorizer, name []byte, keys []string, expr influxql.Expr, fieldset *MeasurementFieldSet) ([]map[string]struct{}, error) {
-	release := is.SeriesFile.Retain()
-	defer release()
-	return is.tagValuesByKeyAndExpr(auth, name, keys, expr)
-}
-
-// tagValuesByKeyAndExpr retrieves tag values for the provided tag keys. See
-// TagValuesByKeyAndExpr for more details.
 //
 // tagValuesByKeyAndExpr guarantees to never take any locks on the underlying
 // series file.
 func (is IndexSet) tagValuesByKeyAndExpr(auth query.Authorizer, name []byte, keys []string, expr influxql.Expr) ([]map[string]struct{}, error) {
 	database := is.Database()
 
-	itr, err := is.seriesByExprIterator(name, expr)
+	valueExpr, remainingExpr, err := influxql.PartitionExpr(influxql.CloneExpr(expr), func(e influxql.Expr) (bool, error) {
+		switch e := e.(type) {
+		case *influxql.BinaryExpr:
+			switch e.Op {
+			case influxql.EQ, influxql.NEQ, influxql.EQREGEX, influxql.NEQREGEX:
+				tag, ok := e.LHS.(*influxql.VarRef)
+				if ok && tag.Val == "value" {
+					return true, nil
+				}
+			}
+		}
+		return false, nil
+	})
+	if err != nil {
+		return nil, err
+	}
+	if remainingExpr == nil {
+		remainingExpr = &influxql.BooleanLiteral{Val: true}
+	}
+
+	itr, err := is.seriesByExprIterator(name, remainingExpr)
 	if err != nil {
 		return nil, err
 	} else if itr == nil {
@@ -2371,6 +2902,18 @@ func (is IndexSet) tagValuesByKeyAndExpr(auth query.Authorizer, name []byte, key
 			break
 		}
 
+		if e.Expr != nil {
+			// We don't yet have code that correctly processes expressions that
+			// seriesByExprIterator doesn't handle
+			lit, ok := e.Expr.(*influxql.BooleanLiteral)
+			if !ok {
+				return nil, fmt.Errorf("expression too complex for metaquery: %v", e.Expr)
+			}
+			if !lit.Val {
+				continue
+			}
+		}
+
 		buf := is.SeriesFile.SeriesKey(e.SeriesID)
 		if len(buf) == 0 {
 			continue
@@ -2378,17 +2921,28 @@ func (is IndexSet) tagValuesByKeyAndExpr(auth query.Authorizer, name []byte, key
 
 		if auth != nil {
 			name, tags := ParseSeriesKey(buf)
+			if len(name) == 0 {
+				continue
+			}
 			if !auth.AuthorizeSeriesRead(database, name, tags) {
 				continue
 			}
 		}
 
 		_, buf = ReadSeriesKeyLen(buf)
+		if len(buf) == 0 {
+			continue
+		}
 		_, buf = ReadSeriesKeyMeasurement(buf)
 		tagN, buf := ReadSeriesKeyTagN(buf)
 		for i := 0; i < tagN; i++ {
 			var key, value []byte
 			key, value, buf = ReadSeriesKeyTag(buf)
+			if valueExpr != nil {
+				if !influxql.EvalBool(valueExpr, map[string]interface{}{"value": string(value)}) {
+					continue
+				}
+			}
 
 			if idx, ok := keyIdxs[string(key)]; ok {
 				resultSet[idx][string(value)] = struct{}{}
@@ -2591,8 +3145,7 @@ func (is IndexSet) TagSets(sfile *SeriesFile, name []byte, opt query.IteratorOpt
 		if !ok {
 			// This TagSet is new, create a new entry for it.
 			tagSet = &query.TagSet{
-				Tags: nil,
-				Key:  tagsAsKey,
+				Key: tagsAsKey,
 			}
 		}
 
@@ -2622,17 +3175,6 @@ func (is IndexSet) TagSets(sfile *SeriesFile, name []byte, opt query.IteratorOpt
 	return sortedTagsSets, nil
 }
 
-// IndexFormat represents the format for an index.
-type IndexFormat int
-
-const (
-	// InMemFormat is the format used by the original in-memory shared index.
-	InMemFormat IndexFormat = 1
-
-	// TSI1Format is the format used by the tsi1 index.
-	TSI1Format IndexFormat = 2
-)
-
 // NewIndexFunc creates a new index.
 type NewIndexFunc func(id uint64, database, path string, seriesIDSet *SeriesIDSet, sfile *SeriesFile, options EngineOptions) Index
 
