@@ -4,14 +4,17 @@ import (
 	"bytes"
 	"testing"
 
-	"github.com/influxdata/influxdb/models"
-	"github.com/influxdata/influxdb/tsdb/index/tsi1"
+	"github.com/influxdata/influxdb/v2/models"
+	"github.com/influxdata/influxdb/v2/tsdb/index/tsi1"
 )
 
 // Ensure multiple index files can be compacted together.
 func TestIndexFiles_WriteTo(t *testing.T) {
+	sfile := MustOpenSeriesFile(t)
+	defer sfile.Close()
+
 	// Write first file.
-	f0, err := CreateIndexFile([]Series{
+	f0, err := CreateIndexFile(sfile.SeriesFile, []Series{
 		{Name: []byte("cpu"), Tags: models.NewTags(map[string]string{"region": "east"})},
 		{Name: []byte("cpu"), Tags: models.NewTags(map[string]string{"region": "west"})},
 		{Name: []byte("mem"), Tags: models.NewTags(map[string]string{"region": "east"})},
@@ -21,7 +24,7 @@ func TestIndexFiles_WriteTo(t *testing.T) {
 	}
 
 	// Write second file.
-	f1, err := CreateIndexFile([]Series{
+	f1, err := CreateIndexFile(sfile.SeriesFile, []Series{
 		{Name: []byte("cpu"), Tags: models.NewTags(map[string]string{"region": "west"})},
 		{Name: []byte("disk"), Tags: models.NewTags(map[string]string{"region": "east"})},
 	})
@@ -32,14 +35,14 @@ func TestIndexFiles_WriteTo(t *testing.T) {
 	// Compact the two together and write out to a buffer.
 	var buf bytes.Buffer
 	a := tsi1.IndexFiles{f0, f1}
-	if n, err := a.CompactTo(&buf, M, K); err != nil {
+	if n, err := a.CompactTo(&buf, sfile.SeriesFile, M, K, nil); err != nil {
 		t.Fatal(err)
 	} else if n == 0 {
 		t.Fatal("expected data written")
 	}
 
 	// Unmarshal buffer into a new index file.
-	var f tsi1.IndexFile
+	f := tsi1.NewIndexFile(sfile.SeriesFile)
 	if err := f.UnmarshalBinary(buf.Bytes()); err != nil {
 		t.Fatal(err)
 	}
