@@ -6,7 +6,7 @@ import (
 	"fmt"
 	"io"
 
-	"github.com/influxdata/influxdb/tsdb"
+	"github.com/influxdata/influxdb/v2/tsdb"
 )
 
 // LoadFactor is the fill percent for RHH indexes.
@@ -32,7 +32,7 @@ type MeasurementIterator interface {
 }
 
 // MergeMeasurementIterators returns an iterator that merges a set of iterators.
-// Iterators that are first in the list take precendence and a deletion by those
+// Iterators that are first in the list take precedence and a deletion by those
 // early iterators will invalidate elements by later iterators.
 func MergeMeasurementIterators(itrs ...MeasurementIterator) MeasurementIterator {
 	if len(itrs) == 0 {
@@ -180,7 +180,7 @@ func (itr *tsdbTagKeyIteratorAdapter) Next() ([]byte, error) {
 }
 
 // MergeTagKeyIterators returns an iterator that merges a set of iterators.
-// Iterators that are first in the list take precendence and a deletion by those
+// Iterators that are first in the list take precedence and a deletion by those
 // early iterators will invalidate elements by later iterators.
 func MergeTagKeyIterators(itrs ...TagKeyIterator) TagKeyIterator {
 	if len(itrs) == 0 {
@@ -318,7 +318,7 @@ func (itr *tsdbTagValueIteratorAdapter) Next() ([]byte, error) {
 }
 
 // MergeTagValueIterators returns an iterator that merges a set of iterators.
-// Iterators that are first in the list take precendence and a deletion by those
+// Iterators that are first in the list take precedence and a deletion by those
 // early iterators will invalidate elements by later iterators.
 func MergeTagValueIterators(itrs ...TagValueIterator) TagValueIterator {
 	if len(itrs) == 0 {
@@ -544,11 +544,3 @@ func uvarint(data []byte) (value uint64, n int, err error) {
 	}
 	return
 }
-
-// hexdump is a helper for dumping binary data to stderr.
-// func hexdump(data []byte) { os.Stderr.Write([]byte(hex.Dump(data))) }
-
-// stack is a helper for dumping a stack trace.
-// func stack() string {
-// 	return "------------------------\n" + string(debug.Stack()) + "------------------------\n\n"
-// }
