@@ -3,22 +3,21 @@ package tsdb_test
 import (
 	"compress/gzip"
 	"fmt"
-	"io/ioutil"
+	"io"
 	"os"
 	"path/filepath"
 	"reflect"
 	"sync"
 	"testing"
 
-	"github.com/influxdata/influxdb/internal"
-	"github.com/influxdata/influxdb/logger"
-	"github.com/influxdata/influxdb/models"
-	"github.com/influxdata/influxdb/pkg/slices"
-	"github.com/influxdata/influxdb/query"
-	"github.com/influxdata/influxdb/tsdb"
-	"github.com/influxdata/influxdb/tsdb/index/inmem"
-	"github.com/influxdata/influxdb/tsdb/index/tsi1"
+	"github.com/influxdata/influxdb/v2/influxql/query"
+	"github.com/influxdata/influxdb/v2/internal"
+	"github.com/influxdata/influxdb/v2/models"
+	"github.com/influxdata/influxdb/v2/pkg/slices"
+	"github.com/influxdata/influxdb/v2/tsdb"
+	"github.com/influxdata/influxdb/v2/tsdb/index/tsi1"
 	"github.com/influxdata/influxql"
+	"go.uber.org/zap/zaptest"
 )
 
 // Ensure iterator can merge multiple iterators together.
@@ -61,7 +60,7 @@ func TestIndexSet_MeasurementNamesByExpr(t *testing.T) {
 	// Setup indexes
 	indexes := map[string]*Index{}
 	for _, name := range tsdb.RegisteredIndexes() {
-		idx := MustOpenNewIndex(name)
+		idx := MustOpenNewIndex(t, name)
 		idx.AddSeries("cpu", map[string]string{"region": "east"})
 		idx.AddSeries("cpu", map[string]string{"region": "west", "secret": "foo"})
 		idx.AddSeries("disk", map[string]string{"secret": "foo"})
@@ -137,55 +136,92 @@ func TestIndexSet_MeasurementNamesByExpr(t *testing.T) {
 	}
 }
 
-func TestIndexSet_DedupeInmemIndexes(t *testing.T) {
-	testCases := []struct {
-		tsiN    int // Quantity of TSI indexes
-		inmem1N int // Quantity of ShardIndexes proxying the first inmem Index
-		inmem2N int // Quantity of ShardIndexes proxying the second inmem Index
-		uniqueN int // Quantity of total, deduplicated indexes
-	}{
-		{tsiN: 1, inmem1N: 0, uniqueN: 1},
-		{tsiN: 2, inmem1N: 0, uniqueN: 2},
-		{tsiN: 0, inmem1N: 1, uniqueN: 1},
-		{tsiN: 0, inmem1N: 2, uniqueN: 1},
-		{tsiN: 0, inmem1N: 1, inmem2N: 1, uniqueN: 2},
-		{tsiN: 0, inmem1N: 2, inmem2N: 2, uniqueN: 2},
-		{tsiN: 2, inmem1N: 2, inmem2N: 2, uniqueN: 4},
+func TestIndexSet_MeasurementNamesByPredicate(t *testing.T) {
+	// Setup indexes
+	indexes := map[string]*Index{}
+	for _, name := range tsdb.RegisteredIndexes() {
+		idx := MustOpenNewIndex(t, name)
+		idx.AddSeries("cpu", map[string]string{"region": "east"})
+		idx.AddSeries("cpu", map[string]string{"region": "west", "secret": "foo"})
+		idx.AddSeries("disk", map[string]string{"secret": "foo"})
+		idx.AddSeries("mem", map[string]string{"region": "west"})
+		idx.AddSeries("gpu", map[string]string{"region": "east"})
+		idx.AddSeries("pci", map[string]string{"region": "east", "secret": "foo"})
+		indexes[name] = idx
+		defer idx.Close()
 	}
 
-	for _, testCase := range testCases {
-		name := fmt.Sprintf("%d/%d/%d -> %d", testCase.tsiN, testCase.inmem1N, testCase.inmem2N, testCase.uniqueN)
-		t.Run(name, func(t *testing.T) {
-
-			var indexes []tsdb.Index
-			for i := 0; i < testCase.tsiN; i++ {
-				indexes = append(indexes, MustOpenNewIndex(tsi1.IndexName))
+	authorizer := &internal.AuthorizerMock{
+		AuthorizeSeriesReadFn: func(database string, measurement []byte, tags models.Tags) bool {
+			if tags.GetString("secret") != "" {
+				t.Logf("Rejecting series db=%s, m=%s, tags=%v", database, measurement, tags)
+				return false
 			}
-			if testCase.inmem1N > 0 {
-				sfile := MustOpenSeriesFile()
-				opts := tsdb.NewEngineOptions()
-				opts.IndexVersion = inmem.IndexName
-				opts.InmemIndex = inmem.NewIndex("db", sfile.SeriesFile)
+			return true
+		},
+	}
 
-				for i := 0; i < testCase.inmem1N; i++ {
-					indexes = append(indexes, inmem.NewShardIndex(uint64(i), tsdb.NewSeriesIDSet(), opts))
+	type example struct {
+		name     string
+		expr     influxql.Expr
+		expected [][]byte
+	}
+
+	// These examples should be run without any auth.
+	examples := []example{
+		{name: "all", expected: slices.StringsToBytes("cpu", "disk", "gpu", "mem", "pci")},
+		{name: "EQ", expr: influxql.MustParseExpr(`region = 'west'`), expected: slices.StringsToBytes("cpu", "mem")},
+		{name: "NEQ", expr: influxql.MustParseExpr(`region != 'west'`), expected: slices.StringsToBytes("cpu", "disk", "gpu", "pci")},
+		{name: "EQREGEX", expr: influxql.MustParseExpr(`region =~ /.*st/`), expected: slices.StringsToBytes("cpu", "gpu", "mem", "pci")},
+		{name: "NEQREGEX", expr: influxql.MustParseExpr(`region !~ /.*est/`), expected: slices.StringsToBytes("cpu", "disk", "gpu", "pci")},
+		// None of the series have this tag so all should be selected.
+		{name: "EQ empty", expr: influxql.MustParseExpr(`host = ''`), expected: slices.StringsToBytes("cpu", "disk", "gpu", "mem", "pci")},
+		// Measurements that have this tag at all should be returned.
+		{name: "NEQ empty", expr: influxql.MustParseExpr(`region != ''`), expected: slices.StringsToBytes("cpu", "gpu", "mem", "pci")},
+		{name: "EQREGEX empty", expr: influxql.MustParseExpr(`host =~ /.*/`), expected: slices.StringsToBytes("cpu", "disk", "gpu", "mem", "pci")},
+		{name: "NEQ empty", expr: influxql.MustParseExpr(`region !~ /.*/`), expected: slices.StringsToBytes()},
+	}
+
+	// These examples should be run with the authorizer.
+	authExamples := []example{
+		{name: "all", expected: slices.StringsToBytes("cpu", "gpu", "mem")},
+		{name: "EQ", expr: influxql.MustParseExpr(`region = 'west'`), expected: slices.StringsToBytes("mem")},
+		{name: "NEQ", expr: influxql.MustParseExpr(`region != 'west'`), expected: slices.StringsToBytes("cpu", "gpu")},
+		{name: "EQREGEX", expr: influxql.MustParseExpr(`region =~ /.*st/`), expected: slices.StringsToBytes("cpu", "gpu", "mem")},
+		{name: "NEQREGEX", expr: influxql.MustParseExpr(`region !~ /.*est/`), expected: slices.StringsToBytes("cpu", "gpu")},
+		{name: "EQ empty", expr: influxql.MustParseExpr(`host = ''`), expected: slices.StringsToBytes("cpu", "gpu", "mem")},
+		{name: "NEQ empty", expr: influxql.MustParseExpr(`region != ''`), expected: slices.StringsToBytes("cpu", "gpu", "mem")},
+		{name: "EQREGEX empty", expr: influxql.MustParseExpr(`host =~ /.*/`), expected: slices.StringsToBytes("cpu", "gpu", "mem")},
+		{name: "NEQ empty", expr: influxql.MustParseExpr(`region !~ /.*/`), expected: slices.StringsToBytes()},
+	}
+
+	for _, idx := range tsdb.RegisteredIndexes() {
+		t.Run(idx, func(t *testing.T) {
+			t.Run("no authorization", func(t *testing.T) {
+				for _, example := range examples {
+					t.Run(example.name, func(t *testing.T) {
+						names, err := indexes[idx].IndexSet().MeasurementNamesByPredicate(nil, example.expr)
+						if err != nil {
+							t.Fatal(err)
+						} else if !reflect.DeepEqual(names, example.expected) {
+							t.Fatalf("got names: %v, expected %v", slices.BytesToStrings(names), slices.BytesToStrings(example.expected))
+						}
+					})
 				}
-			}
-			if testCase.inmem2N > 0 {
-				sfile := MustOpenSeriesFile()
-				opts := tsdb.NewEngineOptions()
-				opts.IndexVersion = inmem.IndexName
-				opts.InmemIndex = inmem.NewIndex("db", sfile.SeriesFile)
+			})
 
-				for i := 0; i < testCase.inmem2N; i++ {
-					indexes = append(indexes, inmem.NewShardIndex(uint64(i), tsdb.NewSeriesIDSet(), opts))
+			t.Run("with authorization", func(t *testing.T) {
+				for _, example := range authExamples {
+					t.Run(example.name, func(t *testing.T) {
+						names, err := indexes[idx].IndexSet().MeasurementNamesByPredicate(authorizer, example.expr)
+						if err != nil {
+							t.Fatal(err)
+						} else if !reflect.DeepEqual(names, example.expected) {
+							t.Fatalf("got names: %v, expected %v", slices.BytesToStrings(names), slices.BytesToStrings(example.expected))
+						}
+					})
 				}
-			}
-
-			is := tsdb.IndexSet{Indexes: indexes}.DedupeInmemIndexes()
-			if len(is.Indexes) != testCase.uniqueN {
-				t.Errorf("expected %d indexes, got %d", testCase.uniqueN, len(is.Indexes))
-			}
+			})
 		})
 	}
 }
@@ -235,7 +271,7 @@ func TestIndex_Sketches(t *testing.T) {
 	}
 
 	test := func(t *testing.T, index string) error {
-		idx := MustNewIndex(index)
+		idx := MustNewIndex(t, index)
 		if index, ok := idx.Index.(*tsi1.Index); ok {
 			// Override the log file max size to force a log file compaction sooner.
 			// This way, we will test the sketches are correct when they have been
@@ -278,14 +314,7 @@ func TestIndex_Sketches(t *testing.T) {
 		}
 
 		// Check cardinalities after the delete
-		switch idx.Index.(type) {
-		case *tsi1.Index:
-			checkCardinalities(t, idx, "initial|reopen|delete", 2923, 0, 10, 2)
-		case *inmem.ShardIndex:
-			checkCardinalities(t, idx, "initial|reopen|delete", 2430, 486, 10, 2)
-		default:
-			panic("unreachable")
-		}
+		checkCardinalities(t, idx, "initial|reopen|delete", 2430, 486, 10, 2)
 
 		// Re-open step only applies to the TSI index.
 		if _, ok := idx.Index.(*tsi1.Index); ok {
@@ -295,7 +324,7 @@ func TestIndex_Sketches(t *testing.T) {
 			}
 
 			// Check cardinalities after the reopen
-			checkCardinalities(t, idx, "initial|reopen|delete|reopen", 2923, 0, 10, 2)
+			checkCardinalities(t, idx, "initial|reopen|delete|reopen", 2430, 486, 10, 2)
 		}
 		return nil
 	}
@@ -317,20 +346,32 @@ type Index struct {
 	sfile     *tsdb.SeriesFile
 }
 
+type EngineOption func(opts *tsdb.EngineOptions)
+
+// DisableTSICache allows the caller to disable the TSI bitset cache during a test.
+var DisableTSICache = func() EngineOption {
+	return func(opts *tsdb.EngineOptions) {
+		opts.Config.SeriesIDSetCacheSize = 0
+	}
+}
+
 // MustNewIndex will initialize a new index using the provide type. It creates
 // everything under the same root directory so it can be cleanly removed on Close.
 //
 // The index will not be opened.
-func MustNewIndex(index string) *Index {
+func MustNewIndex(tb testing.TB, index string, eopts ...EngineOption) *Index {
+	tb.Helper()
+
 	opts := tsdb.NewEngineOptions()
 	opts.IndexVersion = index
 
-	rootPath, err := ioutil.TempDir("", "influxdb-tsdb")
-	if err != nil {
-		panic(err)
+	for _, opt := range eopts {
+		opt(&opts)
 	}
 
-	seriesPath, err := ioutil.TempDir(rootPath, tsdb.SeriesFileDirectory)
+	rootPath := tb.TempDir()
+
+	seriesPath, err := os.MkdirTemp(rootPath, tsdb.SeriesFileDirectory)
 	if err != nil {
 		panic(err)
 	}
@@ -340,18 +381,11 @@ func MustNewIndex(index string) *Index {
 		panic(err)
 	}
 
-	if index == inmem.IndexName {
-		opts.InmemIndex = inmem.NewIndex("db0", sfile)
-	}
-
 	i, err := tsdb.NewIndex(0, "db0", filepath.Join(rootPath, "index"), tsdb.NewSeriesIDSet(), sfile, opts)
 	if err != nil {
 		panic(err)
 	}
-
-	if testing.Verbose() {
-		i.WithLogger(logger.New(os.Stderr))
-	}
+	i.WithLogger(zaptest.NewLogger(tb))
 
 	idx := &Index{
 		Index:     i,
@@ -364,8 +398,10 @@ func MustNewIndex(index string) *Index {
 
 // MustOpenNewIndex will initialize a new index using the provide type and opens
 // it.
-func MustOpenNewIndex(index string) *Index {
-	idx := MustNewIndex(index)
+func MustOpenNewIndex(tb testing.TB, index string, opts ...EngineOption) *Index {
+	tb.Helper()
+
+	idx := MustNewIndex(tb, index, opts...)
 	idx.MustOpen()
 	return idx
 }
@@ -404,9 +440,6 @@ func (i *Index) Reopen() error {
 
 	opts := tsdb.NewEngineOptions()
 	opts.IndexVersion = i.indexType
-	if i.indexType == inmem.IndexName {
-		opts.InmemIndex = inmem.NewIndex("db0", i.sfile)
-	}
 
 	idx, err := tsdb.NewIndex(0, "db0", filepath.Join(i.rootPath, "index"), tsdb.NewSeriesIDSet(), i.sfile, opts)
 	if err != nil {
@@ -436,7 +469,6 @@ func (i *Index) Close() error {
 //
 // Typical results on an i7 laptop.
 //
-// BenchmarkIndexSet_TagSets/1M_series/inmem-8   	     100	  10430732 ns/op	 3556728 B/op	      51 allocs/op
 // BenchmarkIndexSet_TagSets/1M_series/tsi1-8    	     100	  18995530 ns/op	 5221180 B/op	   20379 allocs/op
 func BenchmarkIndexSet_TagSets(b *testing.B) {
 	// Read line-protocol and coerce into tsdb format.
@@ -457,7 +489,7 @@ func BenchmarkIndexSet_TagSets(b *testing.B) {
 		b.Fatal(err)
 	}
 
-	data, err := ioutil.ReadAll(gzr)
+	data, err := io.ReadAll(gzr)
 	if err != nil {
 		b.Fatal(err)
 	}
@@ -492,18 +524,13 @@ func BenchmarkIndexSet_TagSets(b *testing.B) {
 		}
 	}
 
-	// TODO(edd): refactor how we call into tag sets in the tsdb package.
-	type indexTagSets interface {
-		TagSets(name []byte, options query.IteratorOptions) ([]*query.TagSet, error)
-	}
-
 	var errResult error
 
 	// This benchmark will merge eight bitsets each containing ~10,000 series IDs.
 	b.Run("1M series", func(b *testing.B) {
 		b.ReportAllocs()
 		for _, indexType := range tsdb.RegisteredIndexes() {
-			idx := MustOpenNewIndex(indexType)
+			idx := MustOpenNewIndex(b, indexType)
 			setup(idx)
 
 			name := []byte("m4")
@@ -513,18 +540,8 @@ func BenchmarkIndexSet_TagSets(b *testing.B) {
 				Indexes:    []tsdb.Index{idx.Index},
 			} // For TSI implementation
 
-			var ts func() ([]*query.TagSet, error)
-			// TODO(edd): this is somewhat awkward. We should unify this difference somewhere higher
-			// up than the engine. I don't want to open an engine do a benchmark on
-			// different index implementations.
-			if indexType == tsdb.InmemIndexName {
-				ts = func() ([]*query.TagSet, error) {
-					return idx.Index.(indexTagSets).TagSets(name, opt)
-				}
-			} else {
-				ts = func() ([]*query.TagSet, error) {
-					return indexSet.TagSets(idx.sfile, name, opt)
-				}
+			ts := func() ([]*query.TagSet, error) {
+				return indexSet.TagSets(idx.sfile, name, opt)
 			}
 
 			b.Run(indexType, func(b *testing.B) {
@@ -547,10 +564,10 @@ func BenchmarkIndexSet_TagSets(b *testing.B) {
 // This benchmark concurrently writes series to the index and fetches cached bitsets.
 // The idea is to emphasize the performance difference when bitset caching is on and off.
 //
-// Typical results for an i7 laptop
+// # Typical results for an i7 laptop
 //
-// BenchmarkIndex_ConcurrentWriteQuery/inmem/queries_100000-8         	       1	5866592461 ns/op	2499768464 B/op		23964591 allocs/op
-// BenchmarkIndex_ConcurrentWriteQuery/tsi1/queries_100000-8          	       1	30059490078 ns/op	32582973824 B/op	96705317 allocs/op
+// BenchmarkIndex_ConcurrentWriteQuery/tsi1/queries_100000/cache-8        1	1645048376 ns/op	2215402840 B/op	 23048978 allocs/op
+// BenchmarkIndex_ConcurrentWriteQuery/tsi1/queries_100000/no_cache-8     1	22242155616 ns/op	28277544136 B/op 79620463 allocs/op
 func BenchmarkIndex_ConcurrentWriteQuery(b *testing.B) {
 	// Read line-protocol and coerce into tsdb format.
 	keys := make([][]byte, 0, 1e6)
@@ -570,7 +587,7 @@ func BenchmarkIndex_ConcurrentWriteQuery(b *testing.B) {
 		b.Fatal(err)
 	}
 
-	data, err := ioutil.ReadAll(gzr)
+	data, err := io.ReadAll(gzr)
 	if err != nil {
 		b.Fatal(err)
 	}
@@ -590,8 +607,14 @@ func BenchmarkIndex_ConcurrentWriteQuery(b *testing.B) {
 		tags = append(tags, pt.Tags())
 	}
 
-	runBenchmark := func(b *testing.B, index string, queryN int) {
-		idx := MustOpenNewIndex(index)
+	runBenchmark := func(b *testing.B, index string, queryN int, useTSICache bool) {
+		var idx *Index
+		if !useTSICache {
+			idx = MustOpenNewIndex(b, index, DisableTSICache())
+		} else {
+			idx = MustOpenNewIndex(b, index)
+		}
+
 		var wg sync.WaitGroup
 		begin := make(chan struct{})
 
@@ -649,7 +672,7 @@ func BenchmarkIndex_ConcurrentWriteQuery(b *testing.B) {
 			}
 
 			// Re-open everything
-			idx = MustOpenNewIndex(index)
+			idx = MustOpenNewIndex(b, index)
 			wg.Add(1)
 			begin = make(chan struct{})
 			once = sync.Once{}
@@ -663,7 +686,13 @@ func BenchmarkIndex_ConcurrentWriteQuery(b *testing.B) {
 		b.Run(indexType, func(b *testing.B) {
 			for _, queryN := range queries {
 				b.Run(fmt.Sprintf("queries %d", queryN), func(b *testing.B) {
-					runBenchmark(b, indexType, queryN)
+					b.Run("cache", func(b *testing.B) {
+						runBenchmark(b, indexType, queryN, true)
+					})
+
+					b.Run("no cache", func(b *testing.B) {
+						runBenchmark(b, indexType, queryN, false)
+					})
 				})
 			}
 		})
