@@ -162,8 +162,6 @@ var set *SeriesIDSet
 // BenchmarkSeriesIDSet_Add/10-4         	5000000	       348 ns/op	       0 B/op	       0 allocs/op
 // BenchmarkSeriesIDSet_Add/100-4        	5000000	       373 ns/op	       0 B/op	       0 allocs/op
 // BenchmarkSeriesIDSet_Add/1000-4       	5000000	       342 ns/op	       0 B/op	       0 allocs/op
-//
-//
 func BenchmarkSeriesIDSet_AddMore(b *testing.B) {
 	cardinalities := []uint64{1, 2, 10, 100, 1000, 10000, 100000, 1000000, 10000000}
 
@@ -202,7 +200,6 @@ func BenchmarkSeriesIDSet_AddMore(b *testing.B) {
 // BenchmarkSeriesIDSet_Add/cardinality_1000000_check_add/random_global_lock-8       	 2000000	       914   ns/op	       0 B/op	       0 allocs/op
 // BenchmarkSeriesIDSet_Add/cardinality_1000000_check_add/same_multi_lock-8          	30000000	        39.7 ns/op	       0 B/op	       0 allocs/op
 // BenchmarkSeriesIDSet_Add/cardinality_1000000_check_add/random_multi_lock-8        	 1000000	      1002   ns/op	       0 B/op	       0 allocs/op
-//
 func BenchmarkSeriesIDSet_Add(b *testing.B) {
 	// Setup...
 	set = NewSeriesIDSet()
@@ -212,7 +209,7 @@ func BenchmarkSeriesIDSet_Add(b *testing.B) {
 	lookup := uint64(300032)
 
 	// Add the same value over and over.
-	b.Run(fmt.Sprint("cardinality_1000000_add"), func(b *testing.B) {
+	b.Run("cardinality_1000000_add", func(b *testing.B) {
 		b.Run("same", func(b *testing.B) {
 			for i := 0; i < b.N; i++ {
 				set.Add(lookup)
@@ -245,7 +242,7 @@ func BenchmarkSeriesIDSet_Add(b *testing.B) {
 	})
 
 	// Add the same value over and over with no lock
-	b.Run(fmt.Sprint("cardinality_1000000_check_add"), func(b *testing.B) {
+	b.Run("cardinality_1000000_check_add", func(b *testing.B) {
 		b.Run("same no lock", func(b *testing.B) {
 			for i := 0; i < b.N; i++ {
 				if !set.ContainsNoLock(lookup) {
@@ -523,7 +520,6 @@ func BenchmarkSeriesIDSet_AddMany(b *testing.B) {
 // BenchmarkSeriesIDSet_Remove/cardinality_1000000_remove_same-4         		20000000	        99.1 ns/op	       0 B/op	       0 allocs/op
 // BenchmarkSeriesIDSet_Remove/cardinality_1000000_check_remove_global_lock-4   20000000	        57.7 ns/op	       0 B/op	       0 allocs/op
 // BenchmarkSeriesIDSet_Remove/cardinality_1000000_check_remove_multi_lock-4    20000000	        80.1 ns/op	       0 B/op	       0 allocs/op
-//
 func BenchmarkSeriesIDSet_Remove(b *testing.B) {
 	// Setup...
 	set = NewSeriesIDSet()
@@ -533,7 +529,7 @@ func BenchmarkSeriesIDSet_Remove(b *testing.B) {
 	lookup := uint64(300032)
 
 	// Remove the same value over and over.
-	b.Run(fmt.Sprint("cardinality_1000000_remove_same"), func(b *testing.B) {
+	b.Run("cardinality_1000000_remove_same", func(b *testing.B) {
 		for i := 0; i < b.N; i++ {
 			set.Remove(lookup)
 		}
@@ -541,7 +537,7 @@ func BenchmarkSeriesIDSet_Remove(b *testing.B) {
 
 	// Check if the value exists before adding it. Subsequent repeats of the code
 	// will result in contains checks.
-	b.Run(fmt.Sprint("cardinality_1000000_check_remove_global_lock"), func(b *testing.B) {
+	b.Run("cardinality_1000000_check_remove_global_lock", func(b *testing.B) {
 		for i := 0; i < b.N; i++ {
 			set.Lock()
 			if set.ContainsNoLock(lookup) {
@@ -552,7 +548,7 @@ func BenchmarkSeriesIDSet_Remove(b *testing.B) {
 	})
 
 	// Check if the value exists before adding it under two locks.
-	b.Run(fmt.Sprint("cardinality_1000000_check_remove_multi_lock"), func(b *testing.B) {
+	b.Run("cardinality_1000000_check_remove_multi_lock", func(b *testing.B) {
 		for i := 0; i < b.N; i++ {
 			if set.Contains(lookup) {
 				set.Remove(lookup)
